function [disp0_point,stress0_point]=anal_solu_2D(x,y)

%------------------------------------------------------------------------
%  Purpose:
%     Compute the analytical solutions of displacement and stress
%
%  Synopsis:
%     [disp0_point,stress0_point,strain0_point]=anal_solu_2D(x,y)
%
%  Variable Description:
%   disp0_point    = analytical solution of displacement at point
%   stress0_point  = analytical solution of stress at point 
%   x              = coordinate of point in x-axis
%   y              = coordinate of point in y-axis
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS) (2005-2010)                      %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

%Important note: The authors decided to release the source codes with the hope that the S-FEM technique can be applied to more problems and can be further developed into even more powerful methods. 
% If the code is used in anyway in their work, the users are required to cite the book and the following related original papers of the authors: 

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
%--------------------------------------------------------------------------

global emodule poisson example

switch example
    case 'cantilever'
        global lengthx lengthy fload
        %-----------------------------------------------------------------------
        %  analytical solution:
        %  ux=Py/(6EI)*[(6L-3x)*x+(2+poisson)*(y^2-D^2/4)];
        %  uy=-P/(6EI)*[3*poisson*y^2*(L-x)+(4+5*poisson)*D^2*x/4+(3*L-x)*x^2];
        %-----------------------------------------------------------------------
        inert=lengthy*lengthy*lengthy/12;
        pei=fload/(6*emodule*inert);
        ux=pei*y*((6*lengthx-3*x)*x+(2+poisson)*(y*y-lengthy*lengthy/4));
        uy=-pei*(3*poisson*y*y*(lengthx-x)+(4+5*poisson)*lengthy*lengthy*x/4+(3*lengthx-x)*x*x);
        dux=pei*6*(lengthx-x)*y;
        dvy=-pei*6*poisson*y*(lengthx-x);
        dxy=pei*6*(1+poisson)*(y*y-lengthy*lengthy/4);                % shearing strain: duy+dvx

        sigx=fload*(lengthx-x)*y/inert;
        sigy=0;
        sigxy=fload*(y*y-lengthy*lengthy/4)/2/inert;
        
        disp0_point=[ux;uy];
        strain0_point=[dux;dvy;dxy];
        stress0_point=[sigx;sigy;sigxy];
    case 'platehole'
        global Ra
        G=emodule/(1+poisson)/2;
        a2=Ra*Ra; a4=a2*a2;
        x0=x; y0=y;
        r2=x0*x0+y0*y0; r4=r2*r2;
        r=sqrt(r2);
        s1=y0/r; c1=x0/r;        % note that r is always positive (nonzero)!
        s2=2*s1*c1; c2=2*c1*c1-1;
        c4=c2*c2-s2*s2;
        s4=2*s2*c2;
        ur=((1-2*poisson+c2)+(1+4*(1-poisson)*c2)*a2/r2-a4*c2/r4)*r/(4*G); % for plane strain
        ut=((4*poisson-2)*a2/r2-1-a4/r4)*s2*r/(4*G);
        u0=c1*ur-s1*ut;
        v0=s1*ur+c1*ut;
        sigxx0= 1-a2*(3*c2/2+c4)/r2+3*a4*c4/(2*r4);
        sigyy0= -a2*(0.5*c2-c4)/r2-3*a4*c4/(2*r4);
        sigxy0= -a2*(0.5*s2+s4)/r2+3*a4*s4/(2*r4);     
        disp0_point=[u0;v0];
        stress0_point=[sigxx0;sigyy0;sigxy0];   
end

return