function [K]=cal_K_CSFEM_Q4_selective(nSD)

%------------------------------------------------------------------------
%  Purpose:
%     Calculate the global stiffness matrix K of CSFEM (selective scheme) using Q4 element
%
%  Synopsis:
%     [K]=cal_K_CSFEM_Q4_selective(nSD)  
%
%  Variable Description:
%     K     : global stiffness matrix
%    nSD    : number of smoothing domains per element
%
%  Some main variables in the function
%   matmtx1 : matrix of material constants related to muy-part
%   matmtx2 : matrix of material constants related to lamda-part
%   index   : vector containing system dofs associated with each element
%   K1      : stiffness matrix related to muy-part
%   K2      : stiffness matrix related to lamda-part
%   Ke1     : element stiffness matrix related to muy-part
%   Ke2     : element stiffness matrix related to lamda-part
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Dai KY, Nguyen-Thoi T (2007) A smoothed finite element method for mechanics problems. Computational Mechanics; 39: 859-877.
% Dai KY, Liu GR (2007) Free and forced vibration analysis using the smoothed finite element method (SFEM). Journal of Sound and Vibration; 301: 803-820.
% Dai KY, Liu GR, Nguyen-Thoi T (2007) An n-sided polygonal smoothed finite element method (nSFEM) for solid mechanics. Finite Elements in Analysis and Design; 43: 847?860. 
% Nguyen-Thoi T, Liu GR, Dai KY, Lam KY (2007) Selective Smoothed Finite Element Method. Tsinghua Science and Technology; 12(5): 497-508.
% Liu GR, Nguyen-Thoi T, Dai KY, Lam KY (2007) Theoretical aspects of the smoothed finite element method (SFEM). International Journal for Numerical Methods in Engineering; 71: 902-930.
% Liu GR, Nguyen-Thoi T, Nguyen-Xuan H, Dai KY, Lam KY (2009) On the essence and the evaluation of the shape functions for the smoothed finite element method (SFEM) (Letter to Editor). International Journal for Numerical Methods in Engineering; 77: 1863-1869. 
%--------------------------------------------------------------------------

global sdof ndof nel nnel ele_nods gcoord emodule poisson

muy=emodule/2/(1+poisson);
lamda=2*muy*poisson/(1-2*poisson);
matmtx1=muy*[2 0 0;0 2 0;0 0 1];
matmtx2=lamda*[1 1 0;1 1 0;0 0 0];
K1=sparse(sdof,sdof);     % stiffness matrix related to muy-part 
K2=sparse(sdof,sdof);     % stiffness matrix related to lamda-part
K=sparse(sdof,sdof);	  
for iel=1:nel         %loop for the total number of element        
    for i=1:nnel      % loop for 4 nodes of (iel)-th element
        nod(i)=ele_nods(iel,i);    % extract nodes for (iel)-th element
        xco(i)=gcoord(nod(i),1);   % extract x value of the node
        yco(i)=gcoord(nod(i),2);   % extract y value of the node
    end     
    Ke1=cal_Ke_CSFEM_Q4(nSD,xco,yco,matmtx1);  % element stiffness matrix related to muy-part    
    Ke2=cal_Ke_CSFEM_Q4(1,xco,yco,matmtx2);    % element stiffness matrix related to lamda-part
    index=get_eledof(nod,nnel,ndof);           % extract system dofs for the element       
    K1(index,index) = K1(index,index) + Ke1;       % assemble element stiffness matrices           
    K2(index,index) = K2(index,index) + Ke2;       % assemble element stiffness matrices            
end            

K = K1 + K2;

clear K1 K2 matmtx1 matmtx2 Ke1 Ke2