function [K]=cal_K_ESFEM_T3_aver(edge_data,area_edg,area_T3) 
        
%------------------------------------------------------------------------
%  Purpose:
%     Calculate the global stiffness matrix K of ESFEM using T3 element 
%       by Scheme 2: take average of the adjacent elements of each edge 
%
%  Synopsis:
%     [K]=cal_K_ESFEM_T3_aver(edge_data,area_edg,area_T3) 
%
%  Variable Description:
%     K        = global stiffness matrix
%   edge_data  = matrix containing data of (ied)-th edge 
               % edge_data(ied,1): 1st node of (ied)-th edge
               % edge_data(ied,2): 2nd node of (ied)-th edge
               % edge_data(ied,3): 1st element of (ied)-th edge
               % edge_data(ied,4): 2nd element of (ied)-th edge 
               % (if edge_data(ied,4)==0 
               % => means (ied)-th edge belongs to boundary edges
%   area_edg   = vector containing the area of SD associated with edge
%   area_T3    = vector containing the area of elements
%
%  Some main variables in the function
%   dNdx    = derivative of shape functions w.r.t. physical coordinate x
%   dNdy    = derivative of shape functions w.r.t. physical coordinate y
%   Be      = strain-displacement matrix of elements
%   mat_Be  = matrix containing strain-displacement matrices of elements
%   B_ied   = strain-displacement matrix of (ied)-th smoothing domain (SD)
%   nodB    = nodes contributing (ied)-th smoothing domain
%   index   = vector containing system dofs associated with each SD
%   K_ied   = stiffness matrix of (ied)-th smoothing domain (SD)
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Nguyen-Thoi T, Lam KY (2009) An edge-based smoothed finite element method (ES-FEM) for static, free and forced vibration analyses in solids. Journal of Sound and Vibration; 320: 1100-1130.
% Nguyen-Xuan H, Liu GR, Thai-Hoang C, Nguyen-Thoi T (2009) An edge-based smoothed finite element method with stabilized discrete shear gap technique for analysis of Reissner-Mindlin plates. Computer Methods in Applied Mechanics and Engineering;  199: 471-489.
% Nguyen-Xuan H, Liu GR, Nguyen-Thoi T, Nguyen-Tran C (2009) An edge-based smoothed finite element method for analysis of two-dimensional piezoelectric structures, Smart Materials and Structures 18 (6). no. 065015.
% Nguyen-Thoi T, Liu GR, Vu-Do HC, Nguyen-Xuan H (2009) An edge-based smoothed finite element method (ES-FEM) for visco-elastoplastic analyses of 2D solids using triangular mesh. Computational Mechanics; 45: 23-44.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H (2010) An n-sided polygonal edge-based smoothed finite element method (nES-FEM) for solid mechanics. International Journal for Numerical Methods in Biomedical Engineering; doi:10.1002/cnm.1375.
%--------------------------------------------------------------------------

global sdof nel ele_nods gcoord nnel ndof matmtx

K=sparse(sdof,sdof);		%system matrix
mat_Be=cell(nel,1);
nedge=size(edge_data,1);   % total number of edges
for iel=1:nel       %loop for the total number of element        
    for i=1:nnel    % loop for 4 nodes of (iel)-th element
        nod(i)=ele_nods(iel,i);  % extract nodes for (iel)-th element
        x(i)=gcoord(nod(i),1);   % extract x value of the node
        y(i)=gcoord(nod(i),2);   % extract y value of the node
    end          
    %---------------------------------------
    % find the derivatives of shape functions
    %---------------------------------------	
    dNdx=(1/(2*area_T3(iel)))*[(y(2)-y(3)) (y(3)-y(1)) (y(1)-y(2))];  %derivatives w.r.t x
    dNdy=(1/(2*area_T3(iel)))*[(x(3)-x(2)) (x(1)-x(3)) (x(2)-x(1))];  %derivatives w.r.t y   
    Be=get_Bmat_2D(nnel,dNdx,dNdy);      % strain-displacement matrix of elements                     
    mat_Be{iel}=Be;                      % save into matrix containing strain-displacement m                
end  

for ied = 1:nedge    
    %----------------------------------------------------------------------
    % compute the strain-displacement matrix of (ied)-th SD
    %----------------------------------------------------------------------
    [nodB,B_ied] = get_Bmat_ESFEM_T3_aver(edge_data(ied,:),area_edg(ied),area_T3,mat_Be);    
    K_ied = B_ied' * matmtx * B_ied * area_edg(ied);    % stiffness matrix of (ied)-th SD        
    index = get_eledof(nodB,length(nodB),ndof);    % extract system dofs for (ied)-th SD
    K(index,index) = K(index,index) + K_ied;       % assemble stiffness matrices of SDs       
    clear B_ied nodB K_ied index
end 

clear mat_Be