function [K]=cal_K_ESFEM_T3_selective(edge_data,area_T3,area_edg)

%------------------------------------------------------------------------
%  Purpose:
%     Calculate the global stiffness matrix K of ESFEM_T3 (selective scheme) using T3 element
%
%  Synopsis:
%     [K]=cal_K_ESFEM_T3_selective(edge_data,area_T3,area_edg)  
%
%  Variable Description:
%     K        = global stiffness matrix
%   edge_data  = matrix containing data of (ied)-th edge 
               % edge_data(ied,1): 1st node of (ied)-th edge
               % edge_data(ied,2): 2nd node of (ied)-th edge
               % edge_data(ied,3): 1st element of (ied)-th edge
               % edge_data(ied,4): 2nd element of (ied)-th edge 
               % (if edge_data(ied,4)==0 
               % => means (ied)-th edge belongs to boundary edges
%   area_T3    = vector containing area of each triangular element
%   area_edg   = vector containing area of SD associated with each edge
%
%  Some main variables in the function
%   matmtx1 : matrix of material constants related to muy-part
%   matmtx2 : matrix of material constants related to lamda-part
%   index   : vector containing system dofs associated with each element
%   K1      : stiffness matrix related to muy-part
%   K2      : stiffness matrix related to lamda-part
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Nguyen-Thoi T, Lam KY (2009) An edge-based smoothed finite element method (ES-FEM) for static, free and forced vibration analyses in solids. Journal of Sound and Vibration; 320: 1100-1130.
% Nguyen-Xuan H, Liu GR, Thai-Hoang C, Nguyen-Thoi T (2009) An edge-based smoothed finite element method with stabilized discrete shear gap technique for analysis of Reissner-Mindlin plates. Computer Methods in Applied Mechanics and Engineering;  199: 471-489.
% Nguyen-Xuan H, Liu GR, Nguyen-Thoi T, Nguyen-Tran C (2009) An edge-based smoothed finite element method for analysis of two-dimensional piezoelectric structures, Smart Materials and Structures 18 (6). no. 065015.
% Nguyen-Thoi T, Liu GR, Vu-Do HC, Nguyen-Xuan H (2009) An edge-based smoothed finite element method (ES-FEM) for visco-elastoplastic analyses of 2D solids using triangular mesh. Computational Mechanics; 45: 23-44.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H (2010) An n-sided polygonal edge-based smoothed finite element method (nES-FEM) for solid mechanics. International Journal for Numerical Methods in Biomedical Engineering; doi:10.1002/cnm.1375.
%--------------------------------------------------------------------------

global sdof ndof nel nnode nnel ele_nods gcoord emodule poisson

muy=emodule/2/(1+poisson);
lamda=2*muy*poisson/(1-2*poisson);
matmtx1=muy*[2 0 0;0 2 0;0 0 1];
matmtx2=lamda*[1 1 0;1 1 0;0 0 0];
K1=sparse(sdof,sdof);     % system elastic stiffness matrix
K2=sparse(sdof,sdof);     % system elastic stiffness matrix

K=sparse(sdof,sdof);		%system matrix

mat_B=cell(nel,1);
nedge=size(edge_data,1);   % total number of edges
for iel=1:nel       %loop for the total number of element        
    for i=1:nnel    % loop for 4 nodes of (iel)-th element
        nod(i)=ele_nods(iel,i);  % extract nodes for (iel)-th element
        x(i)=gcoord(nod(i),1);   % extract x value of the node
        y(i)=gcoord(nod(i),2);   % extract y value of the node
    end          
    %---------------------------------------
    % find the derivatives of shape functions
    %---------------------------------------	
    dNdx=(1/(2*area_T3(iel)))*[(y(2)-y(3)) (y(3)-y(1)) (y(1)-y(2))];  %derivatives w.r.t x
    dNdy=(1/(2*area_T3(iel)))*[(x(3)-x(2)) (x(1)-x(3)) (x(2)-x(1))];  %derivatives w.r.t y   
    Be=get_Bmat_2D(nnel,dNdx,dNdy);      % strain-displacement matrix of elements                     
    mat_Be{iel} =Be;                      % save into matrix containing strain-displacement m                
end    

for ied = 1:nedge    
    %----------------------------------------------------------------------
    % compute the strain-displacement matrix of (ied)-th SD
    %----------------------------------------------------------------------
    [nodB,B_ied] = get_Bmat_ESFEM_T3_aver(edge_data(ied,:),area_edg(ied),area_T3,mat_Be);    
    K_ied = B_ied' * matmtx1 * B_ied * area_edg(ied);    % stiffness matrix of (ied)-th SD        
    index = get_eledof(nodB,length(nodB),ndof);    % extract system dofs for (ied)-th SD
    K1(index,index) = K1(index,index) + K_ied;       % assemble stiffness matrices of SDs       
    clear B_ied nodB K_ied index
end 

% find adjacent elements of each node
[nod_adjele]=get_nod_adjele;     
% Compute the area of SD associated with node and element areas
[area_nod,area_T3] = cal_area_nod_T3(nod_adjele); 

for ino = 1:nnode    %loop for the total number of node   
    %----------------------------------------------------------------------
    % compute the strain-displacement matrix of (ino)-th SD
    %----------------------------------------------------------------------
    [nodB,B_ino] = get_Bmat_NSFEM_T3_aver(nod_adjele{ino},area_nod(ino),area_T3,mat_Be);               
    K_ino = B_ino'*matmtx2*B_ino*area_nod(ino);     % stiffness matrix of (ino)-th SD        
    index = get_eledof(nodB,length(nodB),ndof);    % extract system dofs for (ino)-th SD
    K2(index,index) = K2(index,index) + K_ino;       % assemble stiffness matrices of SDs       
    clear B_ino nodB K_ino index
end        

K=K1+K2;

clear K1 K2

clear mat_Be nod_adjele area_nod 