function [K]=cal_K_FEM_Q4(nglx,ngly)

%------------------------------------------------------------------------
%  Purpose:
%     Calculate the global stiffness matrix K of FEM using Q4 element
%
%  Synopsis:
%     [K]=cal_K_FEM_Q4(nglx,ngly)  
%
%  Variable Description:
%     K     : global stiffness matrix
%   nglx    : number of Gauss integration points in x-axis
%   ngly    : number of Gauss integration points in y-axis 
%
%  Some main variables in the function
%   dNdr    : derivative of shape functions w.r.t. natural coordinate pxi
%   dNds    : derivative of shape functions w.r.t. natural coordinate eta
%   dNdx    : derivative of shape functions w.r.t. physical coordinate x
%   dNdy    : derivative of shape functions w.r.t. physical coordinate y
%   Bg      : strain-displacement matrix at Gauss points
%   index   : vector containing system dofs associated with each element
%   Ke      : element stiffness matrix
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
%--------------------------------------------------------------------------

global sdof edof ndof nel nnel ele_nods gcoord matmtx

K=sparse(sdof,sdof);		% initialization of global stiffness matrix
[point2,weight2]=get_Gauss_Q4(nglx,ngly); % Gauss integration points & weights
for iel=1:nel       %loop for the total number of element
    for i=1:nnel    % loop for 4 nodes of (iel)-th element
        nod(i)=ele_nods(iel,i);    % extract nodes for (iel)-th element
        xco(i)=gcoord(nod(i),1);   % extract x value of the node
        yco(i)=gcoord(nod(i),2);   % extract y value of the node
    end 
    % Check to ensure Q4 element is a convex polygon
    [angle]=cal_angle_Q4(xco,yco);    
    if angle(1) >=pi | angle(2) >=pi | angle(3) >=pi | angle(4) >=pi 
        temp='quadrilateral element is concave - FEM can not solve'
        break; 
    end    
    Ke=sparse(edof,edof);	   % initialization of element stiffness matrix   
    %---------------------
    %numerical integration
    %---------------------
    for intx=1:nglx
        xg=point2(intx,1);           % sampling point in x-axis
        wtx=weight2(intx,1);         % weight in x-axis
        for inty=1:ngly
            yg=point2(inty,2);       % sampling point in y-axis
            wty=weight2(inty,2);     % weight in y-axis
            [shape,dNdr,dNds]=get_shape_Q4(xg,yg);     % shape functions & derivatives w.r.t natural coordinate                 
            jacob=cal_jacob_2D(nnel,dNdr,dNds,xco,yco);      % Jacobian                    
            detjacob=det(jacob);                            % determinant of Jacobian                               
            invjacob=inv(jacob);                            % inverse of Jacobian matrix                    
            [dNdx,dNdy]=get_dNdx_2D(nnel,dNdr,dNds,invjacob);  % derivatives w.r.t physical coordinate            
            Bg = get_Bmat_2D(nnel,dNdx,dNdy);           % strain-displacement matrix at Gauss points   
            Ke = Ke + Bg'*matmtx*Bg*wtx*wty*detjacob;   % element stiffness matrix                    
        end
    end 
    index = get_eledof(nod,nnel,ndof);    % extract system dofs for the element 
    K(index,index) = K(index,index) + Ke; % assemble element stiffness matrices 
end