function [K]=cal_K_FEM_T3

%------------------------------------------------------------------------
%  Purpose:
%     Calculate the global stiffness matrix K of problem domain using 
%     triangular elements in respect to physical coordinate system
%
%  Synopsis:
%     [K]=cal_K_FEM_T3  
%
%  Some main variables in the function:
%    index - vector containing system dofs associated with each element
%     dNdx - derivative of shape functions w.r.t. physical coordinate x
%     dNdy - derivative of shape functions w.r.t. physical coordinate y
%     Be   - element strain-displacement matrix
%     Ke   - element stiffness matrix
%     K    - global stiffness matrix
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
%--------------------------------------------------------------------------

global ndof sdof edof nel nnel gcoord ele_nods matmtx 

K=sparse(sdof,sdof);		% initialization of global stiffness matrix
for iel=1:nel               % loop on the total number of element
    nod(1)=ele_nods(iel,1);        %1st connected node for (iel)-th element
    nod(2)=ele_nods(iel,2);        %2nd connected node for (iel)-th element 
    nod(3)=ele_nods(iel,3);        %3rd connected node for (iel)-th element    
    x1=gcoord(nod(1),1); y1=gcoord(nod(1),2);     %coord values of 1st node
    x2=gcoord(nod(2),1); y2=gcoord(nod(2),2);     %coord values of 2nd node
    x3=gcoord(nod(3),1); y3=gcoord(nod(3),2);     %coord values of 3rd node    
    index=get_eledof(nod,nnel,ndof);       %extract system dofs for element    
    %----------------------------------------------------------------------
    %find the derivatives of shape functions and element stiffness matrix
    %----------------------------------------------------------------------    
   	area=0.5*(x1*y2+x2*y3+x3*y1-x1*y3-x2*y1-x3*y2); %area of triangle
	area2=area*2;	
    dNdx=(1/area2)*[(y2-y3) (y3-y1) (y1-y2)];    % derivatives w.r.t x
    dNdy=(1/area2)*[(x3-x2) (x1-x3) (x2-x1)];    % derivatives w.r.t y   
 	Be=get_Bmat_2D(nnel,dNdx,dNdy);        % element strain-disp matrix   
   	Ke=Be'*matmtx*Be*area;                 % element stiffness matrix
    K(index,index)=K(index,index)+Ke;      % assemble element matrices                            
end