function [norm_disp,norm_Ereco]=cal_norms_CSFEM_Q4(nglx,ngly,disp,stress_nod)
    
%------------------------------------------------------------------------
%  Purpose:
%   Calculate error norms by CSFEM-Q4 including: 
%           error norm in displacement   
%           recovery error norm in strain energy
%  Synopsis:
%   [norm_disp,norm_Ereco]=cal_norms_CSFEM_Q4(nglx,ngly,disp,stress_nod)  
%
%  Variable Description:
%   norm_disp  = error norm in displacement
%   norm_Ereco = recovery error norm in strain energy
%   nglx       = number of Gauss integration points in x-axis
%   ngly       = number of Gauss integration points in y-axis 
%   disp       = system nodal displacement vector
%   stress_nod = matrix containing stresses of nodes

%  Some main variables in the function:
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Dai KY, Nguyen-Thoi T (2007) A smoothed finite element method for mechanics problems. Computational Mechanics; 39: 859-877.
% Dai KY, Liu GR (2007) Free and forced vibration analysis using the smoothed finite element method (SFEM). Journal of Sound and Vibration; 301: 803-820.
% Dai KY, Liu GR, Nguyen-Thoi T (2007) An n-sided polygonal smoothed finite element method (nSFEM) for solid mechanics. Finite Elements in Analysis and Design; 43: 847?860. 
% Nguyen-Thoi T, Liu GR, Dai KY, Lam KY (2007) Selective Smoothed Finite Element Method. Tsinghua Science and Technology; 12(5): 497-508.
% Liu GR, Nguyen-Thoi T, Dai KY, Lam KY (2007) Theoretical aspects of the smoothed finite element method (SFEM). International Journal for Numerical Methods in Engineering; 71: 902-930.
% Liu GR, Nguyen-Thoi T, Nguyen-Xuan H, Dai KY, Lam KY (2009) On the essence and the evaluation of the shape functions for the smoothed finite element method (SFEM) (Letter to Editor). International Journal for Numerical Methods in Engineering; 77: 1863-1869. 
%--------------------------------------------------------------------------

global nel nnel ele_nods gcoord edof ndof matmtx
     
% initialization of error norms
norm_Ereco=0;
norm_disp=0;

[point2,weight2]=get_Gauss_Q4(nglx,ngly);   % sampling points & weights
invmat=inv(matmtx) ;   % invert of matrix of material constants

for iel=1:nel          % loop for the total number of element
    for i=1:nnel
        nod(i)=ele_nods(iel,i);       % extract nodes for (iel)-th element
        xco(i)=gcoord(nod(i),1);      % extract x value of the node
        yco(i)=gcoord(nod(i),2);      % extract y value of the node
    end  
    index=get_eledof(nod,nnel,ndof);     % extract system dofs for the element 
    for i=1:edof
        edisp(i,1)=disp(index(i));    % element disp
    end     
    %---------------------
    %numerical integration
    %---------------------
    for intx=1:nglx
        x=point2(intx,1);           %sampling point in x-axis
        wtx=weight2(intx,1);        %weight in x-axis
        for inty=1:ngly
            y=point2(inty,2);       %sampling point in y-axis
            wty=weight2(inty,2);    %weight in y-axis            
            [shape,dNdr,dNds]=get_shape_Q4(x,y);        % shape functions and derivatives at sampling point            
            jacob=cal_jacob_2D(nnel,dNdr,dNds,xco,yco); % Jacobian            
            detjacob=det(jacob);                        % determinant of Jacobian   
            
            %--------------------------------------------------------------
            % displacement and stresses at 4 nodes of element
            %--------------------------------------------------------------
            edisp_4nod=zeros(4,2);     
            estress_4nod=zeros(4,3);            
            for i=1:nnel
                edisp_4nod(i,1)=edisp(2*i-1);
                edisp_4nod(i,2)=edisp(2*i);
                estress_4nod(i,:)=stress_nod(:,nod(i));
            end             
            
            %--------------------------------------------------------------
            % disp and recovery stresses at Gauss points by CSFEM-Q4
            %--------------------------------------------------------------
            x_gp=0;
            y_gp=0;
            edisp_gp=zeros(1,2);
            estressRe_gp=zeros(1,3);
            for i=1:nnel    
               % coordinates of Gauss integration point in global coord's
               x_gp = x_gp + shape(i)*xco(i);      
               y_gp = y_gp + shape(i)*yco(i);
               edisp_gp = edisp_gp + shape(i)* edisp_4nod(i,:);
               estressRe_gp = estressRe_gp + shape(i)* estress_4nod(i,:);               
            end      
            
            %--------------------------------------------------------------
            % exact values of disp and stresses at Gauss integration points
            %--------------------------------------------------------------
            [edisp0_gp,estress0_gp]=anal_solu_2D(x_gp,y_gp);
            
            %--------------------------------------------------------------
            % error norms 
            %--------------------------------------------------------------
            norm_disp=norm_disp+(edisp0_gp'-edisp_gp)*(edisp0_gp'-edisp_gp)'*wtx*wty*detjacob;                        
            norm_Ereco = norm_Ereco+1/2*(estress0_gp'-estressRe_gp)*invmat*(estress0_gp'-estressRe_gp)'*wtx*wty*detjacob;
        end
    end   
end

norm_Ereco=sqrt(norm_Ereco);   % recovery error norm in strain energy
norm_disp=sqrt(norm_disp);     % error norm in displacement