function [norm_disp,norm_Ereco]=cal_norms_ESFEM_T3(nGauss,disp,stress_nod,stress_edg,ele_edgs,mm)

%------------------------------------------------------------------------
%  Purpose:
%     Calculate error norms by ESFEM-T3 including: 
%           error norm in displacement   
%           recovery error norm in strain energy
%  Synopsis:
%     [norm_disp,norm_Ereco]=cal_norms_ESFEM_T3(nGauss,disp,stress_nod,stress_edg,ele_edgs,mm)  
%
%  Variable Description:
%   norm_disp  = error norm in displacement
%   norm_Ereco = recovery error norm in strain energy
%   nGauss     = number of Gauss intergration points per each T3 element
%   disp       = system nodal displacement vector
%   stress_nod = matrix containing stresses of nodes
%   stress_edg = matrix containing stresses of edges
%   ele_edgs   = matrix containing edges of each element
%   mm         = 3 : scheme 1 (recovery stress using only 3 nodes of element)
%   mm         = 6 : scheme 2 (recovery stress using 3 nodes of elements and 
%                              3 mid-points of 3 edges of the element) 
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Nguyen-Thoi T, Lam KY (2009) An edge-based smoothed finite element method (ES-FEM) for static, free and forced vibration analyses in solids. Journal of Sound and Vibration; 320: 1100-1130.
% Nguyen-Xuan H, Liu GR, Thai-Hoang C, Nguyen-Thoi T (2009) An edge-based smoothed finite element method with stabilized discrete shear gap technique for analysis of Reissner-Mindlin plates. Computer Methods in Applied Mechanics and Engineering;  199: 471-489.
% Nguyen-Xuan H, Liu GR, Nguyen-Thoi T, Nguyen-Tran C (2009) An edge-based smoothed finite element method for analysis of two-dimensional piezoelectric structures, Smart Materials and Structures 18 (6). no. 065015.
% Nguyen-Thoi T, Liu GR, Vu-Do HC, Nguyen-Xuan H (2009) An edge-based smoothed finite element method (ES-FEM) for visco-elastoplastic analyses of 2D solids using triangular mesh. Computational Mechanics; 45: 23-44.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H (2010) An n-sided polygonal edge-based smoothed finite element method (nES-FEM) for solid mechanics. International Journal for Numerical Methods in Biomedical Engineering; doi:10.1002/cnm.1375.
%--------------------------------------------------------------------------

global nel nnel ele_nods gcoord edof ndof matmtx

% initialization of error norms
norm_Ereco=0;
norm_disp=0;
invmat=inv(matmtx);

[point,weight]=get_Gauss_T3(nGauss);  % Gauss integration points & weights

for iel=1:nel       %loop for the total number of element
    for i=1:nnel
        nod(i)=ele_nods(iel,i);       % extract nodes for (iel)-th element
        xco(i)=gcoord(nod(i),1);      % extract x value of the node
        yco(i)=gcoord(nod(i),2);      % extract y value of the node             
        estress_nod(:,i)=stress_nod(:,nod(i));
    end  
    if mm==6
        xco(4)=(xco(1)+xco(2))/2;     % mid-point of 1st node and 2nd node
        xco(5)=(xco(2)+xco(3))/2;     % mid-point of 2nd node and 3rd node
        xco(6)=(xco(1)+xco(3))/2;     % mid-point of 3rd node and 1st node
        yco(4)=(yco(1)+yco(2))/2;
        yco(5)=(yco(2)+yco(3))/2;
        yco(6)=(yco(1)+yco(3))/2;
        estress_nod(:,4)=stress_edg(:,ele_edgs(iel,1));
        estress_nod(:,5)=stress_edg(:,ele_edgs(iel,2));
        estress_nod(:,6)=stress_edg(:,ele_edgs(iel,3));
    end
    index=get_eledof(nod,nnel,ndof);     % extract system dofs associated with element
    for i=1:edof
        edisp(i,1)=disp(index(i));       % extract element displacement 
    end     
    %---------------------
    %numerical integration
    %---------------------
    for j=1:nGauss
        [shape,dNdr,dNds]=get_shape_T3(point(j,1),point(j,2));  % shape functions & derivatives w.r.t natural coordinat
        jacob=cal_jacob_2D(nnel,dNdr,dNds,xco(1:3),yco(1:3));  % compute Jacobian   
        detjacob=det(jacob);                           % determinant of Jacobian   
        if mm==6
            [shape2,dNdr2,dNds2]=get_shape_T6(point(j,1),point(j,2)); 
            jacob2=cal_jacob_2D(mm,dNdr2,dNds2,xco,yco);  % compute Jacobian   
            detjacob2=det(jacob2);                           % determinant of Jacobian   
        end     
        %------------------------------------------------------------------
        % displacement at 3 nodes of element
        %------------------------------------------------------------------
        xq=0;yq=0;        
        edisp_3nod=zeros(3,2);       % displacement at 3 nodes of element      
        for i=1:nnel
            edisp_3nod(i,1)=edisp(2*i-1);
            edisp_3nod(i,2)=edisp(2*i);            
        end                                                                                                                      
        %------------------------------------------------------------------
        % disp and recovery stresses at Gauss integration points by ESFEM-T3
        %------------------------------------------------------------------
        x_gp=0; y_gp=0;
        edisp_gp=zeros(1,2);       % disp at Gauss integration points 
        estressRe_gp=zeros(3,1);   % recovery stresses at Gauss integration points 
        for i=1:nnel 
            % coordinates of Gauss integration point in global coord's
            x_gp = x_gp + shape(i)*xco(i);      % Gauss integration point in global coord's
            y_gp = y_gp + shape(i)*yco(i);
            edisp_gp = edisp_gp + shape(i)* edisp_3nod(i,:);                        
        end                       
        if mm==3
            for i=1:mm
                estressRe_gp = estressRe_gp + shape(i)* estress_nod(:,i);                                   
            end
        elseif mm==6
            x_gp2=0; y_gp2=0;    
            for i=1:mm
                % coordinates of Gauss integration point in global coord's
                x_gp2 = x_gp2 + shape2(i)*xco(i);      % Gauss integration point in global coord's
                y_gp2 = y_gp2 + shape2(i)*yco(i);
                estressRe_gp = estressRe_gp + shape2(i)* estress_nod(:,i);                                   
            end            
        end
        %------------------------------------------------------------------
        % exact values of disp and stresses at Gauss integration points
        %------------------------------------------------------------------
        [edisp0_gp,estress0_gp]=anal_solu_2D(x_gp,y_gp);
        if mm==6
            [edisp0_gp2,estress0_gp2]=anal_solu_2D(x_gp2,y_gp2);
        end

        %------------------------------------------------------------------
        % error norms 
        %------------------------------------------------------------------
        norm_disp = norm_disp + weight(j,1)*detjacob*(edisp0_gp'-edisp_gp)*(edisp0_gp'-edisp_gp)';                        
        if mm==3
            norm_Ereco = norm_Ereco + 1/2*weight(j,1)*detjacob*(estress0_gp-estressRe_gp)'*invmat*(estress0_gp-estressRe_gp);        
        elseif mm==6
            norm_Ereco = norm_Ereco + 1/2*weight(j,1)*detjacob2*(estress0_gp2-estressRe_gp)'*invmat*(estress0_gp2-estressRe_gp);        
        end
    end   
end

norm_Ereco=sqrt(norm_Ereco);   % recovery error norm in strain enrrgy
norm_disp=sqrt(norm_disp);     % error norm in displacement                     
 