function [norm_disp,norm_E,norm_Ereco]=cal_norms_FEM_Q4(nglx,ngly,disp,stress_nod)
  
%------------------------------------------------------------------------
%  Purpose:
%   Calculate error norms by FEM-Q4 including: 
%           error norm in displacement   
%           error norm in strain enery
%           recovery error norm in strain energy
%  Synopsis:
%   [norm_disp,norm_E,norm_Ereco]=cal_norms_FEM_Q4(nglx,ngly,disp,stress_nod)  
%
%  Variable Description:
%   norm_disp  = error norm in displacement
%   norm_E     = error norm in strain enery
%   norm_Ereco = recovery error norm in strain energy
%   nglx       = number of Gauss integration points in x-axis
%   ngly       = number of Gauss integration points in y-axis 
%   disp       = system nodal displacement vector
%   stress_nod = matrix containing stresses of nodes

%  Some main variables in the function:
%   Bg         = strain-displacement matrix at Gauss points
%   index      = vector containing system dofs associated with each element
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
%--------------------------------------------------------------------------

global nel nnel ele_nods gcoord edof ndof matmtx         
    
% initialization of error norms
norm_Ereco=0;
norm_E=0;
norm_disp=0;
%energy=0;   % strain energy by FEM-Q4 (optional)
%energy0=0;  % exact strain energy     (optional)

[point2,weight2]=get_Gauss_Q4(nglx,ngly);    % sampling points & weights
invmat=inv(matmtx);   % invert of matrix of material constants

for iel=1:nel       %loop for the total number of element
    for i=1:nnel
        nod(i)=ele_nods(iel,i);       % extract nodes for (iel)-th element
        xco(i)=gcoord(nod(i),1);      % extract x value of the node
        yco(i)=gcoord(nod(i),2);      % extract y value of the node
    end  
    index=get_eledof(nod,nnel,ndof);  % extract system dofs for the element 
    for i=1:edof
        edisp(i,1)=disp(index(i));    % element disp 
    end     
    %---------------------
    %numerical integration
    %---------------------
    for intx=1:nglx
        x=point2(intx,1);           %sampling point in x-axis
        wtx=weight2(intx,1);        %weight in x-axis
        for inty=1:ngly
            y=point2(inty,2);       %sampling point in y-axis
            wty=weight2(inty,2);    %weight in y-axis            
            [shape,dNdr,dNds]=get_shape_Q4(x,y);         % shape functions and derivatives at sampling point            
            jacob=cal_jacob_2D(nnel,dNdr,dNds,xco,yco);  % Jacobian            
            detjacob=det(jacob);                         % determinant of Jacobian                       
            invjacob=inv(jacob);                         % inverse of Jacobian matrix            
            [dNdx,dNdy]=get_dNdx_2D(nnel,dNdr,dNds,invjacob);  % derivatives w.r.t physical coordinate            
 	        Bg=get_Bmat_2D(nnel,dNdx,dNdy);               % strain-disp matrix at Gauss points   
            estrain_gp=Bg*edisp;             % straines at Gauss points
            estress_gp=matmtx*estrain_gp;    % stresses at Gauss points
            %--------------------------------------------------------------
            % displacement and stresses at 4 nodes of element
            %--------------------------------------------------------------
            edisp_4nod=zeros(4,2);     % displacement at 4 nodes of element
            estress_4nod=zeros(4,3);   % stresses at 4 nodes of element          
            for i=1:nnel
                edisp_4nod(i,1)=edisp(2*i-1);
                edisp_4nod(i,2)=edisp(2*i);
                estress_4nod(i,:)=stress_nod(:,nod(i));
            end                        
            %--------------------------------------------------------------
            % disp and recovery stresses at Gauss points by FEM-Q4
            %--------------------------------------------------------------
            x_gp=0; y_gp=0;
            edisp_gp=zeros(1,2);      % disp at Gauss integration points
            estressRe_gp=zeros(1,3);  % recovery stresses at Gauss integration points
            for i=1:nnel                               
               % coordinates of Gauss integration point in global coord's
               x_gp = x_gp + shape(i)*xco(i);      
               y_gp = y_gp + shape(i)*yco(i);
               edisp_gp = edisp_gp + shape(i)* edisp_4nod(i,:);
               estressRe_gp = estressRe_gp + shape(i)* estress_4nod(i,:);               
            end
            %--------------------------------------------------------------
            % exact values of disp and stresses at Gauss integration points
            %--------------------------------------------------------------
            [edisp0_gp,estress0_gp]=anal_solu_2D(x_gp,y_gp);
            
            %--------------------------------------------------------------
            % error norms 
            %--------------------------------------------------------------
            %energy = energy + 1/2*wtx*wty*detjacob*estress_gp'*invmat*estress_gp;
            %energy0 = energy0 + 1/2*wtx*wty*detjacob*estress0_gp'*invmat*estress0_gp;
            destress_gp=estress_gp-estress0_gp;
            destressRe_gp=estressRe_gp'-estress0_gp;
            norm_disp=norm_disp+(edisp0_gp'-edisp_gp)*(edisp0_gp'-edisp_gp)'*wtx*wty*detjacob;     
            norm_E = norm_E + 1/2*destress_gp'*invmat*destress_gp*wtx*wty*detjacob;                                  
            norm_Ereco=norm_Ereco+1/2*destressRe_gp'*invmat*destressRe_gp*wtx*wty*detjacob;
        end
    end   
end

%energy0 = sqrt(energy0);         % (optional)
%energy = sqrt(energy);           % (optional)
norm_disp=sqrt(norm_disp);        % error norm in displacement
norm_E=sqrt(norm_E);              % error norm in strain energy
norm_Ereco=sqrt(norm_Ereco);      % recovery error norm in strain energy
