function [norm_disp,norm_E,norm_Ereco]=cal_norms_FEM_T3(nGauss,disp,stress_nod,stress_ele)
    
%------------------------------------------------------------------------
%  Purpose:
%     Calculate error norms by FEM-T3 including: 
%           error norm in displacement   
%           error norm in strain enery
%           recovery error norm in strain energy
%  Synopsis:
%     [norm_disp,norm_E,norm_Ereco]=cal_norms_FEM_T3(nGauss,disp,stress_nod,stress_ele)  
%
%  Variable Description:
%   norm_disp  = error norm in displacement
%   norm_E     = error norm in strain enery
%   norm_Ereco = recovery error norm in strain energy
%   nGauss     = number of Gauss intergration points per each T3 element
%   disp       = system nodal displacement vector
%   stress_nod = matrix containing stresses of nodes
%   stress_ele = matrix containing stresses of elements

%  Some main variables in the function:
%   index      = vector containing system dofs associated with each element
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS) (2005-2010)                      %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
%--------------------------------------------------------------------------

global nel nnel ele_nods gcoord edof ndof matmtx example

% initialization of error norms
norm_Ereco=0;
norm_E=0;
norm_disp=0;  
%energy=0;     % strain energy by FEM-T3 (optional)
%energy0=0;    % exact strain energy     (optional)
invmat=inv(matmtx);

[gpoint,weight]=get_Gauss_T3(nGauss); % Gauss integration points & weights

for iel=1:nel       % loop for the total number of element
    for i=1:nnel
        nod(i)=ele_nods(iel,i);   % extract nodes for (iel)-th element
        xco(i)=gcoord(nod(i),1);  % extract x value of the nodal coordinate
        yco(i)=gcoord(nod(i),2);  % coord values of jst node              
    end        
    index=get_eledof(nod,nnel,ndof); % extract system dofs associated with element
    for i=1:edof
        edisp(i,1)=disp(index(i));   % extract element displacement
    end 
    estress = stress_ele(:,iel);     % element stresses
    %---------------------
    %numerical integration
    %---------------------
    for igp=1:nGauss  
        [shape,dNdr,dNds]=get_shape_T3(gpoint(igp,1),gpoint(igp,2));   % shape functions & derivatives w.r.t natural coordinat
        jacob2D=cal_jacob_2D(nnel,dNdr,dNds,xco,yco);   % compute Jacobian  
        detjacob=det(jacob2D);                         % determinant of Jacobian                                     
        %------------------------------------------------------------------
        % displacement and stresses at 3 nodes of element
        %------------------------------------------------------------------
        edisp_3nod = zeros(3,2);      % displacement at 3 nodes of element
        estress_3nod = zeros(3,3);    % stresses at 3 nodes of element        
        for i=1:nnel
            edisp_3nod(i,1)=edisp(2*i-1);
            edisp_3nod(i,2)=edisp(2*i);
            estress_3nod(i,:)=stress_nod(:,nod(i));
        end                                                                                                                      
        %------------------------------------------------------------------
        % disp and recovery stresses at Gauss integration points by FEM-T3
        %------------------------------------------------------------------
        x_gp=0; y_gp=0;
        edisp_gp=zeros(1,2);       % disp at Gauss integration points        
        estressRe_gp=zeros(1,3);   % recovery stresses at Gauss integration points 
        for i=1:nnel                        
            % coordinates of Gauss integration point in global coord's
            x_gp = x_gp + shape(i)*xco(i);  
            y_gp = y_gp + shape(i)*yco(i);
            edisp_gp = edisp_gp + shape(i)* edisp_3nod(i,:);
            estressRe_gp = estressRe_gp + shape(i)* estress_3nod(i,:);               
        end      
        %------------------------------------------------------------------
        % exact values of disp and stresses at Gauss integration points
        %------------------------------------------------------------------
        [edisp0_gp,estress0_gp]=anal_solu_2D(x_gp,y_gp);
        
        %------------------------------------------------------------------
        % error norms 
        %------------------------------------------------------------------
        %energy = energy + 1/2*weight(igp,1)*detjacob*estress'*invmat*estress;
        %energy0 = energy0 + 1/2*weight(igp,1)*detjacob*estress0_gp'*invmat*estress0_gp;
        destress_gp=estress-estress0_gp;
        destressRe_gp=estressRe_gp'-estress0_gp;
        norm_disp = norm_disp + weight(igp,1)*detjacob*(edisp0_gp'-edisp_gp)*(edisp0_gp'-edisp_gp)';                        
        norm_E = norm_E + 1/2*weight(igp,1)*detjacob*destress_gp'*invmat*destress_gp;   
        norm_Ereco = norm_Ereco + 1/2*weight(igp,1)*detjacob*destressRe_gp'*invmat*destressRe_gp;        
    end   
end

%energy = sqrt(energy);        (optional)
%energy0 = sqrt(energy0);      (optional)
norm_disp  = sqrt(norm_disp);    % error norm in displacement                    
norm_E = sqrt(norm_E);           % error norm in strain energy
norm_Ereco = sqrt(norm_Ereco);   % recovery error norm in strain enrrgy
 