function [norm_disp,norm_Ereco]=cal_norms_NSFEM_T3(nGauss,disp,stress_nod)
 
%------------------------------------------------------------------------
%  Purpose:
%     Calculate error norms by NSFEM-T3 including: 
%           error norm in displacement   
%           recovery error norm in strain energy
%  Synopsis:
%     [norm_disp,norm_Ereco]=cal_norms_NSFEM_T3(nGauss,disp,stress_nod)  
%
%  Variable Description:
%   norm_disp  = error norm in displacement
%   norm_Ereco = recovery error norm in strain energy
%   nGauss     = number of Gauss intergration points per each T3 element
%   disp       = system nodal displacement vector
%   stress_nod = matrix containing stresses of nodes

%  Some main variables in the function:
%   index      = vector containing system dofs associated with each element
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS) (2005-2010)                      %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Nguyen-Thoi T, Nguyen-Xuan H, Lam KY (2009) A node-based smoothed finite element method (NS-FEM) for upper bound solutions to solid mechanics problems. Computers and Structures; 87: 14-26.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H (2009) Additional properties of the node-based smoothed finite element method (NS-FEM) for solid mechanics problems. International Journal of Computational Methods, 6(4): 633-666.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H, Nguyen-Tran C (2010) Adaptive analysis using the node-based smoothed finite element method (NS-FEM). International Journal for Numerical Methods in Biomedical Engineering; in press, doi: 10.1002/cnm.1291.
%--------------------------------------------------------------------------

global nel nnel ele_nods gcoord edof ndof matmtx example

% initialization of error norms
norm_Ereco=0;
norm_disp=0;  
invmat=inv(matmtx);

[gpoint,weight]=get_Gauss_T3(nGauss); % Gauss integration points & weights

for iel=1:nel       % loop for the total number of element
    for i=1:nnel
        nod(i)=ele_nods(iel,i);   % extract nodes for (iel)-th element
        xco(i)=gcoord(nod(i),1);  % extract x value of the nodal coordinate
        yco(i)=gcoord(nod(i),2);  % coord values of jst node              
    end        
    index=get_eledof(nod,nnel,ndof); % extract system dofs associated with element
    for i=1:edof
        edisp(i,1)=disp(index(i));   % extract element displacement
    end 
    %---------------------
    %numerical integration
    %---------------------
    for igp=1:nGauss  
        [shape,dNdr,dNds]=get_shape_T3(gpoint(igp,1),gpoint(igp,2));   % shape functions & derivatives w.r.t natural coordinat
        jacob2D=cal_jacob_2D(nnel,dNdr,dNds,xco,yco);   % compute Jacobian  
        detjacob=det(jacob2D);                         % determinant of Jacobian                                     
        %------------------------------------------------------------------
        % displacement and stresses at 3 nodes of element
        %------------------------------------------------------------------
        edisp_3nod = zeros(3,2);      % displacement at 3 nodes of element
        estress_3nod = zeros(3,3);    % stresses at 3 nodes of element        
        for i=1:nnel
            edisp_3nod(i,1)=edisp(2*i-1);
            edisp_3nod(i,2)=edisp(2*i);
            estress_3nod(i,:)=stress_nod(:,nod(i));
        end                                                                                                                      
        %------------------------------------------------------------------
        % disp and recovery stresses at Gauss integration points by NSFEM-T3
        %------------------------------------------------------------------
        x_gp=0; y_gp=0;
        edisp_gp=zeros(1,2);       % disp at Gauss integration points        
        estressRe_gp=zeros(1,3);   % recovery stresses at Gauss integration points 
        for i=1:nnel                        
            % coordinates of Gauss integration point in global coord's
            x_gp = x_gp + shape(i)*xco(i);  
            y_gp = y_gp + shape(i)*yco(i);
            edisp_gp = edisp_gp + shape(i)* edisp_3nod(i,:);
            estressRe_gp = estressRe_gp + shape(i)* estress_3nod(i,:);               
        end      
        %------------------------------------------------------------------
        % exact values of disp and stresses at Gauss integration points
        %------------------------------------------------------------------
        [edisp0_gp,estress0_gp]=anal_solu_2D(x_gp,y_gp);
        
        %------------------------------------------------------------------
        % error norms 
        %------------------------------------------------------------------
        destressRe_gp=(estressRe_gp'-estress0_gp);
        norm_disp = norm_disp + weight(igp,1)*detjacob*(edisp0_gp'-edisp_gp)*(edisp0_gp'-edisp_gp)';                        
        norm_Ereco = norm_Ereco + 1/2*weight(igp,1)*detjacob*destressRe_gp'*invmat*destressRe_gp;        
    end   
end

norm_disp  = sqrt(norm_disp);    % error norm in displacement                    
norm_Ereco = sqrt(norm_Ereco);   % recovery error norm in strain enrrgy