function [stress_nod,stress_ele]=cal_stress_FEM_T3(disp)

%------------------------------------------------------------------------
%  Purpose:
%     Calculate stresses of all nodes and the stresses of all T3 elements   
%
%  Synopsis:
%     [stress_nod,stress_ele]=cal_stress_FEM_T3(disp)  
%
%  Variable Description:
%   stress_nod = matrix containing stresses of nodes
%   stress_ele = matrix containing stresses of elements
%   disp       = system nodal displacement vector
%
%  Some main variables in the function:
%   index      = vector containing system dofs associated with each element
%   dNdx       = derivative of shape functions w.r.t. physical coordinate x
%   dNdy       = derivative of shape functions w.r.t. physical coordinate y
%   Be         = element strain-displacement matrix
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
%--------------------------------------------------------------------------

global nnode nel nnel ele_nods gcoord edof ndof matmtx

area_ele=zeros(nel,1);

%--------------------------------------------------------------------------
% matrix containing stresses of elements
%--------------------------------------------------------------------------
for iel=1:nel           % loop for the total number of elements
    nod(1)=ele_nods(iel,1); % 1st connected node for (iel)-th element
    nod(2)=ele_nods(iel,2); % 2nd connected node for (iel)-th element
    nod(3)=ele_nods(iel,3); % 3rd connected node for (iel)-th element
    xco(1)=gcoord(nod(1),1); yco(1)=gcoord(nod(1),2);% coord values of 1st node
    xco(2)=gcoord(nod(2),1); yco(2)=gcoord(nod(2),2);% coord values of 2nd node
    xco(3)=gcoord(nod(3),1); yco(3)=gcoord(nod(3),2);% coord values of 3rd node        
    index=get_eledof(nod,nnel,ndof); % extract system dofs associated with element
    for i=1:edof
        edisp(i,1)=disp(index(i));   %  extract element displacement vector
    end
    area=0.5*(xco(1)*yco(2)+xco(2)*yco(3)+xco(3)*yco(1)- ...
                xco(1)*yco(3)-xco(2)*yco(1)-xco(3)*yco(2));  % area of triangle    
    area_ele(iel,1)=area;    
    area2=area*2;
    dNdx=(1/area2)*[(yco(2)-yco(3)) (yco(3)-yco(1)) (yco(1)-yco(2))];  % derivatives w.r.t. x-axis
    dNdy=(1/area2)*[(xco(3)-xco(2)) (xco(1)-xco(3)) (xco(2)-xco(1))];  % derivatives w.r.t. y-axis
 	Be=get_Bmat_2D(nnel,dNdx,dNdy);   % element strain-disp matrix   
    estrain=Be*edisp;                % strains of (iel)-th element
    estress=matmtx*estrain;          % stresses of (iel)-th element
    stress_ele(:,iel)=estress(:);    % store stresses for each element
end

%--------------------------------------------------------------------------
% matrix containing stresses of nodes
%--------------------------------------------------------------------------
stress_nod=zeros(3,nnode);    % initialization of nodal stress matrix
[nod_adjele]=get_nod_adjele;  % find adjacent elements of each node
for ino=1:nnode    % loop for all nodes 
    area_inode=0;  % area of all elements associated with (ino)-th node
    for iel=1:size(nod_adjele{ino},2)
        stress_nod(:,ino)= stress_nod(:,ino)+ stress_ele(:,nod_adjele{ino}(iel))* ...
                                              area_ele(nod_adjele{ino}(iel),1);
        area_inode = area_inode + area_ele(nod_adjele{ino}(iel),1);                                 
    end
    stress_nod(:,ino)= stress_nod(:,ino)/area_inode;
end
