function  [stress_nSD,area_nSD]=cal_stress_nSD_CSFEM_Q4(nSD,xe,ye,disp,nod)

%------------------------------------------------------------------------
%  Purpose:
%     Calculate stresses and areas of nSD smoothing domains in each element
%
%  Synopsis:
%     [stress_nSD,area_nSD]=cal_stress_nSD_CSFEM_Q4(nSD,xe,ye,disp,nod)  
%
%  Variable Description:
%   stress_nSD = stresses of nSD smoothing domains in each element
%   area_nSD   = areas of nSD smoothing domains in each element
%   nSD        = number of smoothing domains per element
%   xe         = coordinate of nodes of the element in x-axis
%   ye         = coordinate of nodes of the element in y-axis
%   disp       = system nodal displacement vector
%   nod        = index of 4 nodes of the element
%
%  Some main variables in the function
%  ecoord_SD : coordinates of points of smoothing domains (SD) in element
%  epoint_SD : connectivity of points of SDs in the element
%  eshape_SD : values of shape functions at points of SDs in the element
%  dN        : derivative of shape functions w.r.t. physical coordinates
%  B_isd     : strain-displacement matrix of (isd)-th SD
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Dai KY, Nguyen-Thoi T (2007) A smoothed finite element method for mechanics problems. Computational Mechanics; 39: 859-877.
% Dai KY, Liu GR (2007) Free and forced vibration analysis using the smoothed finite element method (SFEM). Journal of Sound and Vibration; 301: 803-820.
% Dai KY, Liu GR, Nguyen-Thoi T (2007) An n-sided polygonal smoothed finite element method (nSFEM) for solid mechanics. Finite Elements in Analysis and Design; 43: 847?860. 
% Nguyen-Thoi T, Liu GR, Dai KY, Lam KY (2007) Selective Smoothed Finite Element Method. Tsinghua Science and Technology; 12(5): 497-508.
% Liu GR, Nguyen-Thoi T, Dai KY, Lam KY (2007) Theoretical aspects of the smoothed finite element method (SFEM). International Journal for Numerical Methods in Engineering; 71: 902-930.
% Liu GR, Nguyen-Thoi T, Nguyen-Xuan H, Dai KY, Lam KY (2009) On the essence and the evaluation of the shape functions for the smoothed finite element method (SFEM) (Letter to Editor). International Journal for Numerical Methods in Engineering; 77: 1863-1869. 
%--------------------------------------------------------------------------

global nnel matmtx

edisp=zeros(8,1);
stress_nSD=zeros(3,nSD); area_nSD=zeros(1,nSD);
for i=1:nnel
    edisp(2*i-1)=disp(2*nod(i)-1);
    edisp(2*i)=disp(2*nod(i));
end

%--------------------------------------------------------------------------
% data of coordinates, point connectivity and values of shape function
%-------------------------------------------------------------------------
if nSD==1
  ecoord_SD=[xe(1) ye(1);xe(2) ye(2);xe(3) ye(3);xe(4) ye(4)]; 
  epoint_SD=[1 2 3 4];  
  eshape_SD=[1 0 0 0; 0 1 0 0; 0 0 1 0; 0 0 0 1];  
elseif nSD==2
  ecoord_SD=[xe(1) ye(1);xe(2) ye(2);xe(3) ye(3);xe(4) ye(4);1/2*(xe(1)+xe(2)) 1/2*(ye(1)+ye(2));...
    1/2*(xe(3)+xe(4)) 1/2*(ye(3)+ye(4))];
  epoint_SD=[1 5 6 4; 5 2 3 6];  
  eshape_SD=[1 0 0 0; 0 1 0 0; 0 0 1 0; 0 0 0 1; 0.5 0.5 0 0; 0 0 0.5 0.5];    
elseif nSD==3
  ecoord_SD=[xe(1) ye(1);xe(2) ye(2);xe(3) ye(3);xe(4) ye(4); ...
       1/2*xe(1)+1/2*xe(2) 1/2*ye(1)+1/2*ye(2); 1/4*xe(1)+1/4*xe(2)+1/4*xe(3)+1/4*xe(4)  1/4*ye(1)+1/4*ye(2)+1/4*ye(3)+1/4*ye(4); ...
       1/2*xe(2)+1/2*xe(3) 1/2*ye(2)+1/2*ye(3); 1/2*xe(3)+1/2*xe(4) 1/2*ye(3)+1/2*ye(4)];             
  epoint_SD=[1 5 8 4;5 2 7 6;6 7 3 8];
  eshape_SD=[1 0 0 0;0 1 0 0;0 0 1 0;0 0 0 1;1/2 1/2 0 0;1/4 1/4 1/4 1/4; 0 1/2 1/2 0;0 0 1/2 1/2];      
elseif nSD==4
  ecoord_SD=[xe(1) ye(1);xe(2) ye(2);xe(3) ye(3);xe(4) ye(4);1/2*(xe(1)+xe(2)) 1/2*(ye(1)+ye(2));...
    1/2*(xe(2)+xe(3)) 1/2*(ye(2)+ye(3));1/2*(xe(3)+xe(4)) 1/2*(ye(3)+ye(4));...
    1/2*(xe(4)+xe(1)) 1/2*(ye(4)+ye(1));...
    1/4*(xe(1)+xe(2)+xe(3)+xe(4)) 1/4*(ye(1)+ye(2)+ye(3)+ye(4))];
  epoint_SD=[1 5 9 8;5 2 6 9;9 6 3 7;8 9 7 4];  
  eshape_SD=[1 0 0 0; 0 1 0 0; 0 0 1 0; 0 0 0 1; 0.5 0.5 0 0; 0 0.5 0.5 0; 0 0 0.5 0.5; ...
      0.5 0 0 0.5; 0.25 0.25 0.25 0.25];
elseif nSD==8
       ecoord_SD=[xe(1) ye(1);xe(2) ye(2);xe(3) ye(3);xe(4) ye(4); ...
        1/2*(xe(1)+xe(2)) 1/2*(ye(1)+ye(2)); 1/2*(xe(2)+xe(3)) 1/2*(ye(2)+ye(3));
        1/2*(xe(3)+xe(4)) 1/2*(ye(3)+ye(4)); 1/2*(xe(4)+xe(1)) 1/2*(ye(4)+ye(1));
        3/4*xe(1)+1/4*xe(2) 3/4*ye(1)+1/4*ye(2); 1/4*xe(1)+3/4*xe(2) 1/4*ye(1)+3/4*ye(2);
        3/4*xe(4)+1/4*xe(3) 3/4*ye(4)+1/4*ye(3); 1/4*xe(4)+3/4*xe(3) 1/4*ye(4)+3/4*ye(3);
        3/4*(1/2*(xe(4)+xe(1)))+1/4*(1/2*(xe(2)+xe(3))) 3/4*(1/2*(ye(4)+ye(1)))+1/4*(1/2*(ye(2)+ye(3)));
        1/4*(xe(1)+xe(2)+xe(3)+xe(4)) 1/4*(ye(1)+ye(2)+ye(3)+ye(4));
        1/4*(1/2*(xe(4)+xe(1)))+3/4*(1/2*(xe(2)+xe(3))) 1/4*(1/2*(ye(4)+ye(1)))+3/4*(1/2*(ye(2)+ye(3)))];
    epoint_SD=[1 9 13 8;9 5 14 13;5 10 15 14;10 2 6 15;15 6 3 12;14 15 12 7;13 14 7 11;8 13 11 4];
    eshape_SD=[1 0 0 0;0 1 0 0;0 0 1 0;0 0 0 1;1/2 1/2 0 0;0 1/2 1/2 0;0 0 1/2 1/2;1/2 0 0 1/2;...
       3/4 1/4 0 0;1/4 3/4 0 0;0 0 1/4 3/4;0 0 3/4 1/4;3/8 1/8 1/8 3/8; 1/4 1/4 1/4 1/4;1/8 3/8 3/8 1/8]; 
elseif nSD==16
     ecoord_SD=[xe(1) ye(1);3/4*xe(1)+1/4*xe(2) 3/4*ye(1)+1/4*ye(2);...
       1/2*xe(1)+1/2*xe(2) 1/2*ye(1)+1/2*ye(2); 1/4*xe(1)+3/4*xe(2) 1/4*ye(1)+3/4*ye(2);...
       xe(2) ye(2); 3/4*xe(1)+1/4*xe(4) 3/4*ye(1)+1/4*ye(4);...
       9/16*xe(1)+3/16*xe(2)+3/16*xe(4)+1/16*xe(3) 9/16*ye(1)+3/16*ye(2)+3/16*ye(4)+1/16*ye(3);...
       3/8*xe(1)+3/8*xe(2)+1/8*xe(4)+1/8*xe(3) 3/8*ye(1)+3/8*ye(2)+1/8*ye(4)+1/8*ye(3);...
       9/16*xe(2)+3/16*xe(1)+3/16*xe(3)+1/16*xe(4) 9/16*ye(2)+3/16*ye(1)+3/16*ye(3)+1/16*ye(4);...
       3/4*xe(2)+1/4*xe(3) 3/4*ye(2)+1/4*ye(3);...
       1/2*xe(1)+1/2*xe(4) 1/2*ye(1)+1/2*ye(4);...
       3/8*xe(1)+3/8*xe(4)+1/8*xe(2)+1/8*xe(3) 3/8*ye(1)+3/8*ye(4)+1/8*ye(2)+1/8*ye(3);...
       1/4*xe(1)+1/4*xe(2)+1/4*xe(3)+1/4*xe(4) 1/4*ye(1)+1/4*ye(2)+1/4*ye(3)+1/4*ye(4);...
       3/8*xe(2)+3/8*xe(3)+1/8*xe(1)+1/8*xe(4) 3/8*ye(2)+3/8*ye(3)+1/8*ye(1)+1/8*ye(4);...
       1/2*xe(2)+1/2*xe(3) 1/2*ye(2)+1/2*ye(3);...
       3/4*xe(4)+1/4*xe(1) 3/4*ye(4)+1/4*ye(1);...
       9/16*xe(4)+3/16*xe(1)+3/16*xe(3)+1/16*xe(2) 9/16*ye(4)+3/16*ye(1)+3/16*ye(3)+1/16*ye(2);...
       3/8*xe(4)+3/8*xe(3)+1/8*xe(1)+1/8*xe(2) 3/8*ye(4)+3/8*ye(3)+1/8*ye(1)+1/8*ye(2);...
       9/16*xe(3)+3/16*xe(4)+3/16*xe(2)+1/16*xe(1) 9/16*ye(3)+3/16*ye(4)+3/16*ye(2)+1/16*ye(1);...
       3/4*xe(3)+1/4*xe(2) 3/4*ye(3)+1/4*ye(2);...
       xe(4) ye(4); 3/4*xe(4)+1/4*xe(3) 3/4*ye(4)+1/4*ye(3);...
       1/2*xe(4)+1/2*xe(3) 1/2*ye(4)+1/2*ye(3);...
       3/4*xe(3)+1/4*xe(4) 3/4*ye(3)+1/4*ye(4);...
       xe(3) ye(3)];            
    epoint_SD=[1 2 7 6;2 3 8 7;7 8 13 12;6 7 12 11;...
       3 4 9 8;4 5 10 9;9 10 15 14;8 9 14 13;...
       13 14 19 18;14 15 20 19;19 20 25 24;18 19 24 23;...    
       11 12 17 16;12 13 18 17;17 18 23 22;16 17 22 21];
    eshape_SD=[1 0 0 0;3/4 1/4 0 0;1/2 1/2 0 0;1/4 3/4 0 0;0 1 0 0;...
        3/4 0 0 1/4; 9/16 3/16 1/16 3/16;3/8 3/8 1/8 1/8;3/16 9/16 3/16 1/16;0 3/4 1/4 0;...
        1/2 0 0 1/2;3/8 1/8 1/8 3/8;1/4 1/4 1/4 1/4;1/8 3/8 3/8 1/8;0 1/2 1/2 0;...
        1/4 0 0 3/4; 3/16 1/16 3/16 9/16;1/8 1/8 3/8 3/8;1/16 3/16 9/16 3/16;0 1/4 3/4 0;...
        0 0 0 1;0 0 1/4 3/4;0 0 1/2 1/2;0 0 3/4 1/4;0 0 1 0];  
else
    'Wrong number of sub-domain...'
    pause
end

for isd=1:nSD            % loop on nSD smoothing domains    
    dN=zeros(nnel,2);    % derivative of shape functions on x- and y- axis
    point_isd=epoint_SD(isd,:);   % distract 4 points of (isd)-th SD
    
    % coordinates of 4 nodes of (isd)-th SD   
    for i=1:length(epoint_SD(isd,:))
        ino=point_isd(i);             % i'th connected node of (isd)-th SD
        xsd(i)=ecoord_SD(ino,1); ysd(i)=ecoord_SD(ino,2);     %coord values of i-th point
    end   
    
    %----------------------------------------------------------------------
    % length and normal direction of 4 edges of (isd)-th SD
    %----------------------------------------------------------------------
    [area_isd]=cal_area(xsd,ysd);    % area of (isd)-th SD
    if area_isd<1e-13 
        continue 
    end    
    [ledge_isd]=cal_ledge(xsd,ysd);        % length of 4 edges of (isd)-th SD
    [nx,ny]=cal_nx_ny(xsd,ysd,ledge_isd);  % normal direction of 4 edges of (isd)-th SD
    
    %----------------------------------------------------------------------
    % derivative of shape functions w.r.t. physical coordinates
    %----------------------------------------------------------------------
    for ied=1:4   %loop for 4 edges of quadrilateral SD
     % value of shape function at mid-point on (ied)-th edge of (isd)-th SD
        p1=point_isd(ied); p2=point_isd(mod(ied,4)+1);
        phi=(eshape_SD(p1,:)+eshape_SD(p2,:))/2;
        dN(1,1)=dN(1,1)+phi(1)*nx(ied)*ledge_isd(ied)/area_isd;
        dN(1,2)=dN(1,2)+phi(1)*ny(ied)*ledge_isd(ied)/area_isd;
        dN(2,1)=dN(2,1)+phi(2)*nx(ied)*ledge_isd(ied)/area_isd;
        dN(2,2)=dN(2,2)+phi(2)*ny(ied)*ledge_isd(ied)/area_isd;     
        dN(3,1)=dN(3,1)+phi(3)*nx(ied)*ledge_isd(ied)/area_isd;
        dN(3,2)=dN(3,2)+phi(3)*ny(ied)*ledge_isd(ied)/area_isd;     
        dN(4,1)=dN(4,1)+phi(4)*nx(ied)*ledge_isd(ied)/area_isd;
        dN(4,2)=dN(4,2)+phi(4)*ny(ied)*ledge_isd(ied)/area_isd;   
    end
    
    %----------------------------------------------------------------------
    % strain-displacement matrix of (isd)-th SD 
    %----------------------------------------------------------------------
    B_isd=[];
    for i=1:nnel  %loop for 4 nodes of (isd)-th SD
        B_isd=[B_isd;dN(i,1) 0 dN(i,2);0 dN(i,2) dN(i,1)];
    end 
    
    %----------------------------------------------------------------------
    % stresses and areas of nSD smoothing domains in each element
    %----------------------------------------------------------------------
    strain_isd = B_isd'*edisp;       % strains of (isd)-th SD
    stress_isd = matmtx*strain_isd;  % stresses of (isd)-th SD        
    stress_nSD(:,isd)=stress_isd;    % save into stresses of nSD smoothing domains in each element
    area_nSD(isd)=area_isd;          % save into areas of nSD smoothing domains in each element
end



                
                      