function [stress_nod]=cal_stress_nod_CSFEM_Q4(nSD,disp)

%------------------------------------------------------------------------
%  Purpose:
%     Calculate stresses of all nodes in CSFEM-Q4   
%
%  Synopsis:
%     [stress_nod]=cal_stress_nod_CSFEM_Q4(nSD,disp)  
%
%  Variable Description:
%   stress_nod = matrix containing stresses of nodes
%   nSD        = number of smoothing domains (SD) per element
%   disp       = system nodal displacement vector
%
%  Some main variables in the function:
%   stress_nSD = stresses of nSD smoothing domains in each element
%   area_nSD   = areas of nSD smoothing domains in each element
%   area_nod   = vector containing areas of nodes 
%     area of each node is computed by adding the areas of SDs around that node) 
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Dai KY, Nguyen-Thoi T (2007) A smoothed finite element method for mechanics problems. Computational Mechanics; 39: 859-877.
% Dai KY, Liu GR (2007) Free and forced vibration analysis using the smoothed finite element method (SFEM). Journal of Sound and Vibration; 301: 803-820.
% Dai KY, Liu GR, Nguyen-Thoi T (2007) An n-sided polygonal smoothed finite element method (nSFEM) for solid mechanics. Finite Elements in Analysis and Design; 43: 847?860. 
% Nguyen-Thoi T, Liu GR, Dai KY, Lam KY (2007) Selective Smoothed Finite Element Method. Tsinghua Science and Technology; 12(5): 497-508.
% Liu GR, Nguyen-Thoi T, Dai KY, Lam KY (2007) Theoretical aspects of the smoothed finite element method (SFEM). International Journal for Numerical Methods in Engineering; 71: 902-930.
% Liu GR, Nguyen-Thoi T, Nguyen-Xuan H, Dai KY, Lam KY (2009) On the essence and the evaluation of the shape functions for the smoothed finite element method (SFEM) (Letter to Editor). International Journal for Numerical Methods in Engineering; 77: 1863-1869. 
%--------------------------------------------------------------------------

global nnode nel nnel ele_nods gcoord

stress_nod=zeros(3,nnode);
area_nod=zeros(1,nnode);
for iel=1:nel
    for i=1:nnel    % loop for 4 nodes of (iel)-th element
        nod(i)=ele_nods(iel,i);    % extract nodes for (iel)-th element
        xco(i)=gcoord(nod(i),1);   % extract x value of the node
        yco(i)=gcoord(nod(i),2);   % extract y value of the node
    end 
    %----------------------------------------------------------------------
    % stresses and areas of nSD smoothing domains in the (iel)-th element
    %----------------------------------------------------------------------
    [stress_nSD,area_nSD]=cal_stress_nSD_CSFEM_Q4(nSD,xco,yco,disp,nod);    
    
    %----------------------------------------------------------------------
    % loop to compute the stresses and areas at nodes
    %----------------------------------------------------------------------
    % Distribute stress and area of SDs into nodes based on positions of
    % SDs and nodes as shown in this figure
    %----------------------------------------------------------------------
    %  n4-------n3      n4-------n3     n4-------n3    n4-------n3 
    %  |         |      |    |    |     |    |SD3 |    |SD4 |SD3 |   ...
    %  |   SD1   |      |SD1 | SD2|     |SD1 |-----    |----|----|   ...
    %  |         |      |    |    |     |    |SD2 |    |SD1 |SD2 |   ...  
    %  n1-------n2      n1-------n2     n1-------n2    n1-------n2
    %----------------------------------------------------------------------
    % in case of
    % SD=1 => area and stresses of SD is distributed to 4 nodes of element
    % SD=2 => area and stress of SD1 is distributed to node 1 and node 4
    %         area and stress of SD1 is distributed to node 2 and node 3
    % SD=3 => area and stress of SD1 is distributed to node 1 and node 4
    %         area and stress of SD2 is distributed to node 2 
    %         area and stress of SD3 is distributed to node 3  
    % SD=4 => area and stress of SDi is distributed to node i ...
    %----------------------------------------------------------------------
    for i=1:nnel
        ino=nod(i);
        if nSD==1   
           area_nod(ino)=area_nod(ino)+area_nSD(1)/4; 
           stress_nod(:,ino)=stress_nod(:,ino)+stress_nSD(:,1)*area_nSD(1)/4;          
        elseif nSD==2   
            if i==1 || i==4
                area_nod(ino)=area_nod(ino)+area_nSD(1)/2;  
                stress_nod(:,ino)=stress_nod(:,ino)+stress_nSD(:,1)*area_nSD(1)/2;          
            elseif i==2 || i==3
                area_nod(ino)=area_nod(ino)+area_nSD(2)/2;  
                stress_nod(:,ino)=stress_nod(:,ino)+stress_nSD(:,2)*area_nSD(2)/2;          
            end                
        elseif nSD==3
            if i==1 || i==4
                area_nod(ino)=area_nod(ino)+area_nSD(1)/2;  
                stress_nod(:,ino)=stress_nod(:,ino)+stress_nSD(:,1)*area_nSD(1)/2;         
            elseif i==2 || i==3
                area_nod(ino)=area_nod(ino)+area_nSD(i);  
                stress_nod(:,ino)=stress_nod(:,ino)+stress_nSD(:,i)*area_nSD(i);         
            end                
        elseif nSD==4
            area_nod(ino)=area_nod(ino)+area_nSD(i);
            stress_nod(:,ino)=stress_nod(:,ino)+stress_nSD(:,i)*area_nSD(i);          
        elseif nSD==8
            area_nod(ino)=area_nod(ino)+area_nSD(2*i-1)+area_nSD(2*i);  
            stress_nod(:,ino)=stress_nod(:,ino)+stress_nSD(:,2*i-1)*area_nSD(2*i-1)+stress_nSD(:,2*i)*area_nSD(2*i);          
        elseif nSD==16        
            area_nod(ino)=area_nod(ino)+area_nSD(4*i-3)+area_nSD(4*i-2)+area_nSD(4*i-1)+area_nSD(4*i);  
            stress_nod(:,ino)=stress_nod(:,ino) + stress_nSD(:,4*i-3)*area_nSD(4*i-3)+stress_nSD(:,4*i-2)*area_nSD(4*i-2)+...
                stress_nSD(:,4*i-1)*area_nSD(4*i-1)+stress_nSD(:,4*i)*area_nSD(4*i);          
        end
     end  
end

%--------------------------------------------------------------------------
% average the stresses at nodes 
%--------------------------------------------------------------------------
for i=1:nnode
    stress_nod(:,i)=stress_nod(:,i)/area_nod(i);
end