function [stress_nod]=cal_stress_nod_FEM_Q4(nglx,ngly,disp)

%------------------------------------------------------------------------
%  Purpose:
%     Calculate stresses of all nodes in FEM-Q4   
%
%  Synopsis:
%     [stress_nod]=cal_stress_nod_FEM_Q4(nglx,ngly,disp)  
%
%  Variable Description:
%   stress_nod = matrix containing stresses of nodes
%   nglx       = number of Gauss integration points in x-axis
%   ngly       = number of Gauss integration points in y-axis 
%   disp       = system nodal displacement vector
%
%  Some main variables in the function:
%  extra_mat = extrapolation matrix
%   dNdr     = derivative of shape functions w.r.t. natural coordinate pxi
%   dNds     = derivative of shape functions w.r.t. natural coordinate eta
%   index    = vector containing system dofs associated with each element
%   dNdx     = derivative of shape functions w.r.t. physical coordinate x
%   dNdy     = derivative of shape functions w.r.t. physical coordinate y
%   Bg       = strain-displacement matrix at Gauss points
% estress_gp = stress at Gauss points before rearranged counterclockwisely
%estress_gp2 = stress at Gauss points rearranged counterclockwisely 
%nadjele_nod = number of adjacent elements around each node
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
%--------------------------------------------------------------------------

global nnode nel nnel ele_nods gcoord edof ndof matmtx

%--------------------------------------------------------------------------
%  Extrapolation matrix
%--------------------------------------------------------------------------
extra_mat=[1+0.5*sqrt(3) -0.5  1-0.5*sqrt(3) -0.5;
            -0.5  1+0.5*sqrt(3)  -0.5  1-0.5*sqrt(3);
            1-0.5*sqrt(3)  -0.5  1+0.5*sqrt(3) -0.5;
            -0.5  1-0.5*sqrt(3) -0.5  1+0.5*sqrt(3)];

stress_nod=zeros(3,nnode);   % initialization of matrix containing stresses of nodes
%strain_nod=zeros(3,nnode);
nadjele_nod=zeros(nnode,1);  % number of adjacent elements around each node

[point2,weight2]=get_Gauss_Q4(nglx,ngly);    %sampling points & weights
for iel=1:nel                % loop for the total number of element
    for i=1:nnel
        nod(i)=ele_nods(iel,i);       % extract nodes for (iel)-th element
        xco(i)=gcoord(nod(i),1);      % extract x value of the node
        yco(i)=gcoord(nod(i),2);      % extract y value of the node
    end          
    %---------------------
    %numerical integration
    %---------------------
    estress_gp2=cell(1,nglx*ngly);  % stress at Gauss points rearranged in
                                    % the counterclockwise order of 4 nodes
    ino=0;    
    for intx=1:nglx
        x=point2(intx,1);           % sampling point in x-axis
        wtx=weight2(intx,1);        % weight in x-axis        
        for inty=1:ngly
            y=point2(inty,2);       % sampling point in y-axis
            wty=weight2(inty,2);    % weight in y-axis            
            [shape,dNdr,dNds]=get_shape_Q4(x,y);     % shape functions and derivatives at sampling point            
            jacob2=cal_jacob_2D(nnel,dNdr,dNds,xco,yco);    % Jacobian            
            detjacob=det(jacob2);                           % determinant of Jacobian                      
            invjacob=inv(jacob2);                           % inverse of Jacobian matrix            
            [dNdx,dNdy]=get_dNdx_2D(nnel,dNdr,dNds,invjacob);  % derivatives w.r.t physical coordinate            
 	        Bg=get_Bmat_2D(nnel,dNdx,dNdy);   % strain-disp matrix at Gauss points   
            index=get_eledof(nod,nnel,ndof);  % extract system dofs for element            
            for i=1:edof
                edisp(i,1)=disp(index(i)); % extract element displacement
            end                                   
            estrain_gp=Bg*edisp;             % strain at Gauss points before rearranged counterclockwisely
            estress_gp=matmtx*estrain_gp;    % stress at Gauss points before rearranged counterclockwisely
            %--------------------------------------------------------------
            % rearrange stresses at Gauss points counterclockwisely
            %       n4-----n3          n4------n3
            %       | g2 g4 |    =>    | g4  g3 | 
            %       | g1 g3 |    =>    | g1  g2 |
            %       n1-----n2          n1------n2 
            %  before rearranged =>  after rearranged
            %     estress_gp     =>    estress_gp2
            %--------------------------------------------------------------
            ino=ino+1;
            if ino==2
                estress_gp2{1,4}(1:3)=estress_gp(:);            
            elseif ino==3
                estress_gp2{1,2}(1:3)=estress_gp(:);            
            elseif ino==4
                estress_gp2{1,3}(1:3)=estress_gp(:);  
            else
                estress_gp2{1,1}(1:3)=estress_gp(:);  
            end
        end
    end         
      
    %----------------------------------------------------------------------
    % matrix containing stresses of nodes => extrapolate
    %----------------------------------------------------------------------
    for i=1:nnel  % loop for 4 nodes of (iel)-th element      
        temp_stress=zeros(3,1);
        for ngp=1:nglx*ngly   % loop for 4 Gauss integration points
            temp_stress(:) = temp_stress(:) + extra_mat(i,ngp) * estress_gp2{1,ngp}(:);
        end
        stress_nod(:,nod(i)) = stress_nod(:,nod(i)) + temp_stress(:);
        nadjele_nod(nod(i)) = nadjele_nod(nod(i)) + 1;              
    end
end

%--------------------------------------------------------------------------
% average stresses computed for the same node from adjacent elements
%--------------------------------------------------------------------------
for i=1:nnode
    stress_nod(:,i)=stress_nod(:,i)/nadjele_nod(i);  %unweighted average
    %strain_nod(:)=inv(matmtx)*stress_nod(:,1);
end


