function [nodB,B_ied] = get_Bmat_ESFEM_T3_aver(iedge,area_ied,area_T3,mat_Be)

%------------------------------------------------------------------------
%  Purpose:
%     the strain-displacement matrix of (ied)-th SD and nodes contributing
%     (ied)-th smoothing domain
%
%  Synopsis:
%     [nodB,B_ied] = get_Bmat_ESFEM_T3_aver(iedge,area_ied,area_T3,mat_Be) 
%
%  Variable Description:
%   nodB       = nodes contributing (ied)-th smoothing domain
%   B_ied      = strain-displacement matrix of (ied)-th smoothing domain (SD)
%   iedge      = data of (ied)-th edge
%   area_ied   = area of SD associated with (ied)-th edge 
%   area_T3    = vector containing the area of elements
%   mat_Be     = matrix containing strain-displacement matrices of elements
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Nguyen-Thoi T, Lam KY (2009) An edge-based smoothed finite element method (ES-FEM) for static, free and forced vibration analyses in solids. Journal of Sound and Vibration; 320: 1100-1130.
% Nguyen-Xuan H, Liu GR, Thai-Hoang C, Nguyen-Thoi T (2009) An edge-based smoothed finite element method with stabilized discrete shear gap technique for analysis of Reissner-Mindlin plates. Computer Methods in Applied Mechanics and Engineering;  199: 471-489.
% Nguyen-Xuan H, Liu GR, Nguyen-Thoi T, Nguyen-Tran C (2009) An edge-based smoothed finite element method for analysis of two-dimensional piezoelectric structures, Smart Materials and Structures 18 (6). no. 065015.
% Nguyen-Thoi T, Liu GR, Vu-Do HC, Nguyen-Xuan H (2009) An edge-based smoothed finite element method (ES-FEM) for visco-elastoplastic analyses of 2D solids using triangular mesh. Computational Mechanics; 45: 23-44.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H (2010) An n-sided polygonal edge-based smoothed finite element method (nES-FEM) for solid mechanics. International Journal for Numerical Methods in Biomedical Engineering; doi:10.1002/cnm.1375.
%--------------------------------------------------------------------------

global ele_nods

if iedge(4)==0
    ne=1;  % number of adjacent elements of (ied)-th edge => boundary edge
else
    ne=2;  % number of adjacent elements of (ied)-th edge => inner edge
end        

nodB = ele_nods(iedge(3),:); % nodes of 1st adjacent element

if ne==1  % in case of boudary edges
    for j=1:3 % loop for 3 nodes of 1st adjacent element
        B_ied(1:3,2*j-1:2*j)=1/3*area_T3(iedge(3))*mat_Be{iedge(3)}(1:3,2*j-1:2*j);                               
    end 
else   % in case of inner edge (or ne==2)
    for iel=1:ne       % loop for two adjacent elements of (ied)-th edge  
        if iel==1      % initialization of B_ied for 1st adjacent element
            for j=1:3  % loop for 3 nodes of 1st adjacent element
                B_ied(1:3,2*j-1:2*j)=1/3*area_T3(iedge(3))*mat_Be{iedge(3)}(1:3,2*j-1:2*j);                               
            end             
        else       % assemble into matrix 'B_ied' from 2nd adjacent elements
            i0=0;  % number of added new nodes from 2nd adjacent element
            for jj=1:3 % loop for 3 nodes of 2nd adjacent element
                nod=ele_nods(iedge(4),jj); % extract (jj)-th node of 2nd adjacent element => give variable 'nod'
                flag=0;    % flag = 1: 'nod' belongs to the set 'nodB'  
                           % flag = 0: 'nod' doesn't belongs to the set 'nodB'
                for j=1:3 % loop for 3 current nodes of 'nodB' (from 1st adjacent element)
                    if nodB(j)==nod    % if 'nod' is the j-th member of 'nodB'
                                       % => assemble into 'B_ino' at the corresponding current positions
                        B_ied(1:3,2*j-1:2*j)=B_ied(1:3,2*j-1:2*j) + 1/3*area_T3(iedge(4))*mat_Be{iedge(4)}(1:3,2*jj-1:2*jj);
                        flag=1;  break  % after determining 'nod' to be the j-th member of 'nodB'
                                        % => exit the current inmost loop 'for' to consider the next node of
                                        % 2nd adjacent element
                    end
                end
                if flag==0  % 'nod' doesn't belongs to the set 'nodB' => new node
                    i0=i0+1;
                    nodB(3+i0)=nod;   % add 'nod' into 'nodB'
                     % add strain-disp matrix of 'nod' into 'B_ino' at next new positions 
                    B_ied(1:3,2*(3+i0)-1:2*(3+i0))=1/3*area_T3(iedge(4))*mat_Be{iedge(4)}(1:3,2*jj-1:2*jj);
                end
            end             
         end    % of if iel==1
    end
end
   
B_ied=B_ied/area_ied;    % smooth (or average) the matrix B_ied

return
    