function [nodB,B_ied] = get_Bmat_ESFEM_T3_boun(iedge,area_edg)

%------------------------------------------------------------------------
%  Purpose:
%     the strain-displacement matrix of (ied)-th SD and nodes contributing
%     (ied)-th smoothing domain
%
%  Synopsis:
%     [nodB,B_ied] = get_Bmat_ESFEM_T3_boun(iedge,area_edg) 
%
%  Variable Description:
%   nodB       = nodes contributing (ino)-th smoothing domain (SD)
%   B_ied      = strain-displacement matrix of (ied)-th SD
%   iedge      = data of (ied)-th edge 
%   area_ied   = area of SD associated with (ied)-th edge
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Nguyen-Thoi T, Lam KY (2009) An edge-based smoothed finite element method (ES-FEM) for static, free and forced vibration analyses in solids. Journal of Sound and Vibration; 320: 1100-1130.
% Nguyen-Xuan H, Liu GR, Thai-Hoang C, Nguyen-Thoi T (2009) An edge-based smoothed finite element method with stabilized discrete shear gap technique for analysis of Reissner-Mindlin plates. Computer Methods in Applied Mechanics and Engineering;  199: 471-489.
% Nguyen-Xuan H, Liu GR, Nguyen-Thoi T, Nguyen-Tran C (2009) An edge-based smoothed finite element method for analysis of two-dimensional piezoelectric structures, Smart Materials and Structures 18 (6). no. 065015.
% Nguyen-Thoi T, Liu GR, Vu-Do HC, Nguyen-Xuan H (2009) An edge-based smoothed finite element method (ES-FEM) for visco-elastoplastic analyses of 2D solids using triangular mesh. Computational Mechanics; 45: 23-44.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H (2010) An n-sided polygonal edge-based smoothed finite element method (nES-FEM) for solid mechanics. International Journal for Numerical Methods in Biomedical Engineering; doi:10.1002/cnm.1375.
%--------------------------------------------------------------------------

global ele_nods gcoord

if iedge(4)==0
    ne=1;  % number of adjacent elements of (ied)-th edge => boundary edge
else
    ne=2;  % number of adjacent elements of (ied)-th edge => inner edge
end    

for ie=1:ne  % loop for adjacent elements of edge 'ied'    
    n1 = ele_nods(iedge(2+ie),1);
    n2 = ele_nods(iedge(2+ie),2);
    n3 = ele_nods(iedge(2+ie),3);
    xx1 = gcoord(n1,1); yy1 = gcoord(n1,2);
    xx2 = gcoord(n2,1); yy2 = gcoord(n2,2);
    xx3 = gcoord(n3,1); yy3 = gcoord(n3,2);       
    xcen = (gcoord(n1,1) + gcoord(n2,1) + gcoord(n3,1))/3; 
    ycen = (gcoord(n1,2) + gcoord(n2,2) + gcoord(n3,2))/3;
    
    %Calculate the outward normals of the three mid-line
    xL1 = sqrt(((xx1-xcen)^2+(yy1-ycen)^2));
    onx11 = (ycen-yy1)/xL1; ony11 = (xx1-xcen)/xL1;
        
    xL2 = sqrt(((xx2-xcen)^2+(yy2-ycen)^2));
    onx22 = (ycen-yy2)/xL2; ony22 = (xx2-xcen)/xL2;
        
    xL3 = sqrt(((xx3-xcen)^2+(yy3-ycen)^2));
    onx33 = (ycen-yy3)/xL3; ony33 = (xx3-xcen)/xL3;
        
    %----------------------------------------------------------------------
    % coordinates and value of shape functions of 3 nodes of sub-domain associated with (ied)-th edge  
    % and three outward normals of three edges of sub-domain associated with (ied)-th edge 
    % if (iedge(1)==n1 and iedge(2)==n2) or (iedge(1)==n2 and iedge(2)==n1)
    % => sub-domain : n1 - n2 - cen ; 
    %               three outward normals of edges: n1-n2, n2-cen and cen-n1  
    % if (iedge(1)==n2 and iedge(2)==n3) or (iedge(1)==n3 and iedge(2)==n2)
    % => sub-domain : n2 - n3 - cen ; 
    %               three outward normals of edges: n2-n3, n3-cen and cen-n2  
    % if (iedge(1)==n3 and iedge(2)==n1) or (iedge(1)==n1 and iedge(2)==n3)
    % => sub-domain : n3 - n1 - cen ; 
    %               three outward normals of edges: n3-n1, n1-cen and cen-n3  
    %----------------------------------------------------------------------
    %                    n3
    %                   /| \
    %                  / |  \
    %                 /  |   \
    %                /  cen   \
    %               / /     \  \
    %              / /        \ \
    %             n1------------n2
    %----------------------------------------------------------------------
    if (iedge(1) == n1 & iedge(2) == n2) | (iedge(2) == n1 & iedge(1) == n2)
        xysd(1,1) = gcoord(n1,1); xysd(1,2) = gcoord(n1,2);         
        xysd(2,1) = gcoord(n2,1); xysd(2,2) = gcoord(n2,2);         
        xysd(3,1) = xcen; xysd(3,2) = ycen;        
        xL4 = sqrt((xysd(1,1)-xysd(2,1))^2+(xysd(1,2)-xysd(2,2))^2);
        onsd(1,1) = (xysd(2,2)-xysd(1,2))/xL4; onsd(1,2) = (xysd(1,1)-xysd(2,1))/xL4;        
        onsd(2,1)= onx22; onsd(2,2) = ony22;
        onsd(3,1)= -onx11; onsd(3,2) = -ony11;        
        shape_sd=[1 0 0;0 1 0;1/3 1/3 1/3];  
    elseif (iedge(1) == n2 & iedge(2) == n3) | (iedge(2) == n2 & iedge(1) == n3)
        xysd(1,1) = gcoord(n2,1); xysd(1,2) = gcoord(n2,2);         
        xysd(2,1) = gcoord(n3,1); xysd(2,2) = gcoord(n3,2);         
        xysd(3,1) = xcen; xysd(3,2) = ycen;        
        xL4 = sqrt((xysd(1,1)-xysd(2,1))^2+(xysd(1,2)-xysd(2,2))^2);
        onsd(1,1) = (xysd(2,2)-xysd(1,2))/xL4; onsd(1,2) = (xysd(1,1)-xysd(2,1))/xL4;        
        onsd(2,1)= onx33; onsd(2,2) = ony33;
        onsd(3,1)= -onx22; onsd(3,2) = -ony22;        
        shape_sd=[0 1 0;0 0 1;1/3 1/3 1/3];  
    elseif (iedge(1) == n1 & iedge(2) == n3) | (iedge(2) == n1 & iedge(1) == n3)
        xysd(1,1) = gcoord(n3,1); xysd(1,2) = gcoord(n3,2);         
        xysd(2,1) = gcoord(n1,1); xysd(2,2) = gcoord(n1,2);         
        xysd(3,1) = xcen; xysd(3,2) = ycen;        
        xL4 = sqrt((xysd(1,1)-xysd(2,1))^2+(xysd(1,2)-xysd(2,2))^2);
        onsd(1,1) = (xysd(2,2)-xysd(1,2))/xL4; onsd(1,2) = (xysd(1,1)-xysd(2,1))/xL4;        
        onsd(2,1)= onx11; onsd(2,2) = ony11;
        onsd(3,1)= -onx33; onsd(3,2) = -ony33;        
        shape_sd=[0 0 1;1 0 0;1/3 1/3 1/3];  
    end   
        
    for ieg = 1:3  % loop for 3 edges of smoothing domain 
        nx = onsd(ieg,1); ny = onsd(ieg,2);
        p1=ieg; p2=(mod(ieg,3)+1);          % two points at two ends of (ied)-th edge 
        %-----------------------------------------------------------------
        % values of shape function of mid-point of (ieg)-th edge (or Gauss point)
        %-----------------------------------------------------------------
        phi=1/2*(shape_sd(p1,:) + shape_sd(p2,:));
         %------------------------------------------------------------------
        % length of (ieg)-th edge (or weight of Gauss point)
        %------------------------------------------------------------------
        len_ieg = sqrt((xysd(p1,1)-xysd(p2,1))^2+(xysd(p1,2)-xysd(p2,2))^2); 
        %------------------------------------------------------------------
        % compute the strain-displacement matrix of (ied)-th SD and nodes contributing
        %     (ied)-th smoothing domain
        %------------------------------------------------------------------
        % smooth the strain-displacement matrix of (ied)-th SD              
        norm=([nx 0; 0 ny; ny nx])*len_ieg/area_edg; 
        
        if ie==1 & ieg==1  %  initialization of B_ied for 1st edge of 1st adjacent element
            nodB=ele_nods(iedge(3),:);
            nn=3;     % 3 nodes of 1st adjacent element
            for j=1:3 % loop for 3 nodes of 1st adjacent element                
                B_ied(1:3,2*j-1:2*j) = norm*phi(j);
            end
        else  
            i0=0;  % number of added new nodes from 2nd adjacent element
            for jj=1:3   % loop for 3 nodes of current adjacent element     
                nod=ele_nods(iedge(2+ie),jj); % extract (jj)-th node of (ie)-th adjacent element => give variable 'nod'
                flag=0;   % flag=1: 'nod' belongs to the set 'nodB'  
                          % flag=0: 'nod' doesn't belongs to the set 'nodB'
                for j=1:nn  % loop for nn current nodes of 'nodB' 
                   if nodB(j)==nod  % if 'nod' is the j-th member of 'nodB'
                                    % => assemble into 'B_ied' at the corresponding current positions
                      B_ied(1:3,2*j-1:2*j) = B_ied(1:3,2*j-1:2*j) + norm*phi(jj);
                      flag=1;  break   % after determining 'nod' to be the j-th member of 'nodB'
                                        % => exit the current inmost loop 'for' to consider the next node of
                                        % current adjacent element
                    end
                end
                if flag==0  % 'nod' doesn't belongs to the set 'nodB' => new node
                    i0=i0+1;
                    nodB(nn+i0)=nod;  % add 'nod' into 'nodB'
                    B_ied(1:3,2*(nn+i0)-1:2*(nn+i0))=norm*phi(jj);
                 end
            end
            nn=nn+i0;     % current total number of nodes of 'nodB'        
        end
        clear norm len_ieg phi
    end    
end

return
    