function [nodB,B_ino] = get_Bmat_NSFEM_Q4(ino,ino_adjele,area_ino)

%------------------------------------------------------------------------
%  Purpose:
%     the strain-displacement matrix of (ino)-th SD and nodes contributing
%     (ino)-th smoothing domain
%
%  Synopsis:
%     [nodB,B_ino] = get_Bmat_NSFEM_Q4(ino,ino_adjele,area_ino)
%
%  Variable Description:
%   nodB       = nodes contributing (ino)-th smoothing domain (SD)
%   B_ino      = strain-displacement matrix of (ino)-th SD
%   ino        = (ino)-th node or (ino)-th smoothing domain 
%   ino_adjele = adjacent elements of (ino)-th node
%   area_ino   = area of (ino)-th SD 
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Nguyen-Thoi T, Nguyen-Xuan H, Lam KY (2009) A node-based smoothed finite element method (NS-FEM) for upper bound solutions to solid mechanics problems. Computers and Structures; 87: 14-26.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H (2009) Additional properties of the node-based smoothed finite element method (NS-FEM) for solid mechanics problems. International Journal of Computational Methods, 6(4): 633-666.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H, Nguyen-Tran C (2010) Adaptive analysis using the node-based smoothed finite element method (NS-FEM). International Journal for Numerical Methods in Biomedical Engineering; in press, doi: 10.1002/cnm.1291.
%--------------------------------------------------------------------------

global ele_nods gcoord 

nade = length(ino_adjele); % number of adjacent elements of node  

for ie = 1:nade  % loop for adjacent elements of node 'ino'
    n1 = ele_nods(ino_adjele(ie),1);  % 1st node of (ie)-th adjacent element
    n2 = ele_nods(ino_adjele(ie),2);  % 2nd node of (ie)-th adjacent element
    n3 = ele_nods(ino_adjele(ie),3);  % 3rd node of (ie)-th adjacent element
    n4 = ele_nods(ino_adjele(ie),4);  % 4th node of (ie)-th adjacent element
    xx1 = gcoord(n1,1); yy1 = gcoord(n1,2);  % coords of 1st node
    xx2 = gcoord(n2,1); yy2 = gcoord(n2,2);  % coords of 2nd node
    xx3 = gcoord(n3,1); yy3 = gcoord(n3,2);  % coords of 3rd node
    xx4 = gcoord(n4,1); yy4 = gcoord(n4,2);  % coords of 4th node 
    %----------------------------------------------------------------------
    % coords of mid-points on the edges connecting nodes
    %----------------------------------------------------------------------
    xm1 = (gcoord(n1,1)+gcoord(n2,1))/2; ym1 = (gcoord(n1,2)+gcoord(n2,2))/2;
    xm2 = (gcoord(n2,1)+gcoord(n3,1))/2; ym2 = (gcoord(n2,2)+gcoord(n3,2))/2;
    xm3 = (gcoord(n3,1)+gcoord(n4,1))/2; ym3 = (gcoord(n3,2)+gcoord(n4,2))/2;
    xm4 = (gcoord(n4,1)+gcoord(n1,1))/2; ym4 = (gcoord(n4,2)+gcoord(n1,2))/2;
    %----------------------------------------------------------------------
    % coords of central point of the (ie)-th adjacent element
    %----------------------------------------------------------------------
    xm5 = (gcoord(n1,1)+gcoord(n2,1)+gcoord(n3,1)+gcoord(n4,1))/4;
    ym5 = (gcoord(n1,2)+gcoord(n2,2)+gcoord(n3,2)+gcoord(n4,2))/4;
    
    %----------------------------------------------------------------------
    % normals of four lines connecting 4 nodes with the central point
    %----------------------------------------------------------------------
    xL1 = sqrt(((xm1-xm5)^2+(ym1-ym5)^2));
    onx11 = (ym5-ym1)/xL1; ony11 = (xm1-xm5)/xL1;
    onx12 = -onx11; ony12 = -ony11;
    
    xL2 = sqrt(((xm2-xm5)^2+(ym2-ym5)^2));
    onx22 = (ym5-ym2)/xL2; ony22 = (xm2-xm5)/xL2;
    onx23 = -onx22; ony23 = -ony22;
    
    xL3 = sqrt(((xm3-xm5)^2+(ym3-ym5)^2));
    onx33 = (ym5-ym3)/xL3; ony33 = (xm3-xm5)/xL3;
    onx34 = -onx33; ony34 = -ony33;
    
    xL4 = sqrt(((xm4-xm5)^2+(ym4-ym5)^2));
    onx44 = (ym5-ym4)/xL4; ony44 = (xm4-xm5)/xL4;
    onx41 = -onx44; ony41 = -ony44;
   
     %----------------------------------------------------------------------
    % coordinates and value of shape functions of 4 nodes of sub-domain associated with (ino)-th node  
    % and two outward normals of two inner edges 
    % if ino==n1 => sub-domain : m1 - m5 - m4 - n1 ; 
    %               two outward normals of edges: m1-m5 and m5-m4  
    % if ino==n2 => sub-domain : m2 - m5 - m1 - n2
    %               two outward normals of edges: m2-m5 and m5-m1 
    % if ino==n3 => sub-domain : m3 - m5 - m2 - n3
    %               two outward normals of edges: m3-m5 and m5-m2 
    % if ino==n4 => sub-domain : m4 - m5 - m3 - n4
    %               two outward normals of edges: m4-m5 and m5-m3 
    %----------------------------------------------------------------------
    %             n4----m3-----n3
    %             |      |      |
    %             |      |      |
    %             m4----m5-----m2
    %             |      |      |
    %             |      |      |
    %             n1----m1-----n2
    %----------------------------------------------------------------------
    
    if ino == n1
        onsd(1,1)= onx11; onsd(1,2) = ony11;
        onsd(2,1)= onx41; onsd(2,2) = ony41;
        xysd(1,1) = xm1; xysd(1,2) = ym1;
        xysd(2,1) = xm5; xysd(2,2) = ym5;
        xysd(3,1) = xm4; xysd(3,2) = ym4;
        xysd(4,:) = gcoord(ino,:);
        shape_sd=[1/2 1/2 0 0;1/4 1/4 1/4 1/4;1/2 0 0 1/2;1 0 0 0];  
    elseif ino == n2
        onsd(1,1)= onx22; onsd(1,2) = ony22;
        onsd(2,1)= onx12; onsd(2,2) = ony12;
        xysd(1,1) = xm2; xysd(1,2) = ym2;
        xysd(2,1) = xm5; xysd(2,2) = ym5;
        xysd(3,1) = xm1; xysd(3,2) = ym1;
        xysd(4,:) = gcoord(ino,:);
        shape_sd=[0 1/2 1/2 0;1/4 1/4 1/4 1/4;1/2 1/2 0 0;0 1 0 0];   
    elseif ino == n3
        onsd(1,1)= onx33; onsd(1,2) = ony33;
        onsd(2,1)= onx23; onsd(2,2) = ony23;
        xysd(1,1) = xm3; xysd(1,2) = ym3;
        xysd(2,1) = xm5; xysd(2,2) = ym5;
        xysd(3,1) = xm2; xysd(3,2) = ym2;
        xysd(4,:) = gcoord(ino,:);
        shape_sd=[0 0 1/2 1/2;1/4 1/4 1/4 1/4;0 1/2 1/2 0;0 0 1 0];  
    else
        onsd(1,1)= onx44; onsd(1,2) = ony44;
        onsd(2,1)= onx34; onsd(2,2) = ony34;
        xysd(1,1) = xm4; xysd(1,2) = ym4;
        xysd(2,1) = xm5; xysd(2,2) = ym5;
        xysd(3,1) = xm3; xysd(3,2) = ym3;
        xysd(4,:) = gcoord(ino,:);
        shape_sd=[1/2 0 0 1/2;1/4 1/4 1/4 1/4;0 0 1/2 1/2;0 0 0 1];  
    end
    
    %----------------------------------------------------------------------
    % two outward normals of two outer edges 
    %----------------------------------------------------------------------
    xL4 = sqrt((xysd(4,1)-xysd(3,1))^2+(xysd(4,2)-xysd(3,2))^2);
    onsd(3,1) = (xysd(4,2)-xysd(3,2))/xL4; onsd(3,2) = (xysd(3,1)-xysd(4,1))/xL4;    
    xL5 = sqrt((xysd(4,1)-xysd(1,1))^2+(xysd(4,2)-xysd(1,2))^2);
    onsd(4,1) = (xysd(1,2)-xysd(4,2))/xL5; onsd(4,2) = (xysd(4,1)-xysd(1,1))/xL5;
    
    ie_nods = ele_nods(ino_adjele(ie),:);  % nodes of (ie)-th adjacent element
    nn_ie = length(ie_nods);     % number of nodes of (ie)-th adjacent element
    
    for ied = 1:4  % loop for 4 edges of sub-domain 
        nx = onsd(ied,1); ny = onsd(ied,2); % outward normals of (ied)-th edge
        p1=ied; p2=(mod(ied,4)+1);          % two points at two ends of (ied)-th edge 
        %-----------------------------------------------------------------
        % values of shape function of mid-point of (ied)-th edge (or Gauss point)
        %-----------------------------------------------------------------
        phi=1/2*(shape_sd(p1,:) + shape_sd(p2,:));
        %------------------------------------------------------------------
        % length of (ied)-th edge (or weight of Gauss point)
        %------------------------------------------------------------------
        len_ied = sqrt((xysd(p1,1)-xysd(p2,1))^2+(xysd(p1,2)-xysd(p2,2))^2);               
        %------------------------------------------------------------------
        % compute the strain-displacement matrix of (ino)-th SD and nodes contributing
        %     (ino)-th smoothing domain
        %------------------------------------------------------------------
        % smooth the strain-displacement matrix of (ino)-th SD
        norm=([nx 0; 0 ny; ny nx])*len_ied/area_ino;   
        
        if ie==1 & ied==1  %  initialization of B_ino for 1st edge of 1st adjacent element
             nodB=ele_nods(ino_adjele(ie),:);  % extract nodes of 1st adjacent element 
             nn=nn_ie;                         % current total number of nodes of 'nodB'
            for j=1:nn       % loop for current total number of nodes of 'nodB'                
                B_ino(1:3,2*j-1:2*j)=norm*phi(j);
            end
        else  % assemble into matrix 'B_ino' after 1st edge of 1st adjacent element
            i0=0;  % number of added new nodes from each edge of adjacent elements
            for in=1:nn_ie   % loop for nodes of (ie)-th adjacent element
                nod=ele_nods(ino_adjele(ie),in);   % extract (in)-th node of (ie)-th adjacent element => give variable 'nod'
                flag=0;      % flag=1: 'nod' belongs to the set 'nodB'  
                             % flag=0: 'nod' doesn't belongs to the set 'nodB'      
                for j=1:nn   % loop for current total number of nodes of 'nodB'
                   if nodB(j)==nod      % if 'nod' is the j-th member of 'nodB'    
                                        % => assemble into 'B_ino' at the corresponding current positions                 
                      B_ino(1:3,2*j-1:2*j)=B_ino(1:3,2*j-1:2*j)+norm*phi(in);
                      flag=1;  break  % after determining 'nod' to be the j-th member of 'nodB'
                                      % => exit the current inmost loop 'for' to consider the next node of
                                      % (ie)-th adjacent element
                    end
                end
                if flag==0   % 'nod' doesn't belongs to the set 'nodB' => new node
                    i0=i0+1;
                    nodB(nn+i0)=nod;    % add 'nod' into 'nodB'                
                    % add strain-disp matrix of 'nod' into 'B_ino' at next new positions 
                    B_ino(1:3,2*(nn+i0)-1:2*(nn+i0))=norm*phi(in);
                 end
            end
             nn=nn+i0;     % current total number of nodes of 'nodB'         
        end
    end   
    clear norm phi len_ied
end
return
    