function [gcoord,ele_nods,bcdof,bcval,ff]=get_initdata

%------------------------------------------------------------------------
%  Purpose:
%     Create initial data of geometry, boundary condition and force vector
%
%  Synopsis:
%     [gcoord,ele_nods,bcdof,bcval,ff]=get_initdata
%
%  Variable Description:
%   gcoord     = matrix containing coordinate values of each node
%   ele_nods   = matrix containing nodes of each element  
%   bcdof      = vector containing dofs associated with boundary conditions
%   bcval      = vector containing boundary condition values associated with dofs in bcdof
%   ff         = system force vector
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------

global nnel sdof example

switch example 
    case 'cantilever'     
        global lengthx lengthy nx ny fload air
        %--------------------------------------------
        %input data for nodal coordinate values
        %gcoord(i,j) where i->node no.  and j->x or y
        %--------------------------------------------
        dx=lengthx/nx;          % the length of side of elements in x-axis
        dy=lengthy/ny;          % the length of side of elements in y-axis
        gcoord=[];
        for i=1:nx+1
            for j=ny/2:-1:1
                gcoord=[gcoord; (i-1)*dx -j*dy;]; 
            end
            for j=1:ny/2+1
                gcoord=[gcoord; (i-1)*dx (j-1)*dy;]; 
            end
        end
        nn=0;
        for ip=1:nx+1                % sampling node coordiantes for discretisation
            for iq=1:ny+1 
                nn=nn+1;   
                r=random('beta',1,1);        % r=[0 1];
                r=air*(2*r-1);               % project r=[-air air];
                if ip==1 | ip==nx+1 | iq==1 | iq==ny+1 %| (iq-1)*bb==5 %|(ip-1)*aa==L/2
                    r=0;
                end
                gcoord(nn,1)=gcoord(nn,1)+dx*r;
                gcoord(nn,2)=gcoord(nn,2)+dy*r;  
            end
        end
        %---------------------------------------------------------
        %input data for nodal connectivity for each element
        %ele_nods(i,j) where i->element no. and j->connected nodes
        %---------------------------------------------------------
        ele_nods=[];
        if nnel==4       % for 4-node quadrilateral elements
            for i=1:nx
                for j=1:ny
                    ele_nods=[ele_nods; (ny+1)*(i-1)+j (ny+1)*i+j (ny+1)*i+j+1 (ny+1)*(i-1)+j+1;];
                end
            end
        elseif nnel==3   % for 3-node triangular elements
            for i=1:nx
                for j=1:ny
                    ele_nods=[ele_nods; (ny+1)*(i-1)+j (ny+1)*i+j (ny+1)*i+j+1;(ny+1)*(i-1)+j (ny+1)*i+j+1 (ny+1)*(i-1)+j+1;];
                end
            end
        end
        %----------------------------------
        %input data for boundary conditions
        %----------------------------------
        bcdof=[];
        bcval=[];
        for i=1:ny+1
            bcdof=[bcdof 1+2*(i-1) 2+2*(i-1)];
            xco=gcoord(i,1);
            yco=gcoord(i,2);   
            [pdisp0,pstress0]=anal_solu_2D(xco,yco);
            bcval=[bcval pdisp0(1) pdisp0(2)];
        end
        %--------------------------------------
        %initialization of matrices and vectors
        %--------------------------------------
        ff=sparse(sdof,1);			%system force vector
        %--------------------------------------------------------------------------
        %force vector
        %--------------------------------------------------------------------------
        iner=lengthy^3/12;
        syms h
        for i=2:ny    
            gy=gcoord((ny+1)*nx+i,2);
            temp=double(-int(fload/(2*iner)*(lengthy^2/4-h^2),h,gy-dy/2,gy+dy/2));
            ff((ny+1)*2*nx+(i-1)*2+2)=temp;
        end
        gy1=gcoord((ny+1)*nx+1,2);
        ff((ny+1)*2*nx+2)=double(-int(fload/(2*iner)*(lengthy^2/4-h^2),h,gy1,gy1+dy/2));
        gyny=gcoord((ny+1)*nx+ny+1,2);
        ff((ny+1)*2*nx+ny*2+2)=double(-int(fload/(2*iner)*(lengthy^2/4-h^2),h,gyny-dy/2,gyny));
    
    case 'platehole'
        global nx ny Ra A
        %--------------------------------------------
        %input data for nodal coordinate values
        %gcoord(i,j) where i->node no.  and j->x or y
        %--------------------------------------------
        [gcoord]=create_node_platehole(nx,ny,Ra,A);

        %-----------------------------------------------------
        %input data for nodal connectivity for each element
        %ele_nods(i,j) where i->element no. and j->connected nodes
        %------------------------------------------------------

        ele_nods=[];
        if nnel==4
            for i=1:nx
                for j=1:ny
                    ele_nods=[ele_nods; (ny+1)*(i-1)+j+1 (ny+1)*i+j+1 (ny+1)*i+j (ny+1)*(i-1)+j;];
                end
            end
        elseif nnel==3
            for i=1:nx/2
                for j=1:ny
                    ele_nods=[ele_nods; (ny+1)*(i-1)+j+1 (ny+1)*i+j+1 (ny+1)*i+j; (ny+1)*(i-1)+j+1 (ny+1)*i+j (ny+1)*(i-1)+j;];
                end   
            end
            for i=nx/2+1:nx
                for j=1:ny        
                    ele_nods=[ele_nods; (ny+1)*(i-1)+j+1 (ny+1)*i+j+1 (ny+1)*(i-1)+j; (ny+1)*i+j+1 (ny+1)*i+j (ny+1)*(i-1)+j;];
                end
            end
        end

        %---------------------------------------------------------------------
        %  input data for boundary conditions for displacement and potential
        %---------------------------------------------------------------------
        for i=1:ny+1
            bcdof(1,i)=i*2-1;                    % left edge: in x direction
            bcdof(1,ny+1+i)=((ny+1)*nx+i)*2;     % bottom edge: in y direction  
        end
        bcval=zeros(1,length(bcdof));        % whose described values are 0

        %---------------------------------------------------------------------
        %  initial data of load
        %------------------------------------------------------------------
        [ff]=get_force_platehole(nx,ny,Ra,gcoord(:,1),gcoord(:,2));   
 end