%---------------------------------------------------------------------%
% Main file of 2D problems using triangular or quadrilateral elements %   
%---------------------------------------------------------------------%
%
% Variable descriptions
%   ndof       = number of degrees of freedom (dofs) per node
%   sdof       = total dofs in system
%   edof       = dofs per element
%   nel        = total number of elements in system 
%   nnel       = number of nodes per element
%   nnode      = total number of nodes in system
%   nSD        = number of smoothing domains per element
%   alpha      = parameter alpha used for alpha-FEM models
%   nglx       = number of Gauss integration points in x-axis
%   ngly       = number of Gauss integration points in y-axis 
%   nglxy      = number of Gauss intergration points per each Q4 element
%   nGauss     = number of Gauss intergration points per each T3 element
%   emodule    = Young elastic modulus
%   poisson    = Poisson's ratio
%   matmtx     = matrix of material constants
%   gcoord     = matrix containing coordinate values of each node
%   ele_nods   = matrix containing nodes of each element  
%   ele_edgs   = matrix containing edges of each element
%   nod_adjele = matrix containing adjacent elements of each node
%   edg_adjele = matrix containing adjacent elements of each edge
%   nod_adjedg = matrix containing adjacent edges of each node
%   area_nod   = vector containing area of SD associated with each node
%   area_edg   = vector containing area of SD associated with each edge
%   edge_data  = matrix containing data of (ied)-th edge 
               % edge_data(ied,1): 1st node of (ied)-th edge
               % edge_data(ied,2): 2nd node of (ied)-th edge
               % edge_data(ied,3): 1st element of (ied)-th edge
               % edge_data(ied,4): 2nd element of (ied)-th edge 
               % (if edge_data(ied,4)==0 
               % => means (ied)-th edge belongs to boundary edges
%   area_T3    = vector containing area of each triangular element
%   area_Q4    = vector containing area of each quadrilateral element
%   K          = system stiffness matrix
%   ff         = system force vector
%   disp       = system nodal displacement vector
%   bcdof      = vector containing dofs associated with boundary conditions
%   bcval      = vector containing boundary condition values associated with dofs in bcdof
%   stress_ele = matrix containing stresses of elements
%   stress_nod = matrix containing stresses of nodes
%   stress_edg = matrix containing stresses of edges
%   energy     = strain energy of the problem
%   norm_disp  = error norm in displacement 
%   norm_E     = error norm in strain energy
%   norm_Ereco = recovery error norm in strain energy
%--------------------------------------------------------------------------
% Coded by Dr. Nguyen Thoi Trung (Nguyen-Thoi T or Nguyen T.T)            %
% University of Science - Vietnam National University  HCMC, Vietnam     %
% National University of Singapore (NUS)                                  %
% email: thoitrung76@gmail.com                                            %
% Last modified: December 2009                                            %
%--------------------------------------------------------------------------
 
% Important note: The authors decided to release the source codes free of charge with the hope that the S-FEM technique 
% can be applied to more problems and can be further developed into even more powerful methods. 
% The authors are not be able to provide any services or user-guides, but appreciate very much your feedback on errors and suggestions, 
% so that we can improve these codes/methods in the future. If the idea, method, and any part of these codes are used in anyway,
% the users are required to cite the book and the following related original papers of the authors:

% Liu GR, The Finite element method  a practical course, Elsevier (BH), UK.  2003.
% Liu, G. R. and Nguyen Thoi Trung, Smoothed Finite Element Method, CRC press, Boca Raton, USA, 2010.
% Liu GR, Dai KY, Nguyen-Thoi T (2007) A smoothed finite element method for mechanics problems. Computational Mechanics; 39: 859-877.
% Dai KY, Liu GR (2007) Free and forced vibration analysis using the smoothed finite element method (SFEM). Journal of Sound and Vibration; 301: 803-820.
% Dai KY, Liu GR, Nguyen-Thoi T (2007) An n-sided polygonal smoothed finite element method (nSFEM) for solid mechanics. Finite Elements in Analysis and Design; 43: 847?860. 
% Nguyen-Thoi T, Liu GR, Dai KY, Lam KY (2007) Selective Smoothed Finite Element Method. Tsinghua Science and Technology; 12(5): 497-508.
% Liu GR, Nguyen-Thoi T, Dai KY, Lam KY (2007) Theoretical aspects of the smoothed finite element method (SFEM). International Journal for Numerical Methods in Engineering; 71: 902-930.
% Liu GR, Nguyen-Thoi T, Nguyen-Xuan H, Dai KY, Lam KY (2009) On the essence and the evaluation of the shape functions for the smoothed finite element method (SFEM) (Letter to Editor). International Journal for Numerical Methods in Engineering; 77: 1863-1869. 
% Liu GR, Nguyen-Thoi T, Nguyen-Xuan H, Lam KY (2009) A node-based smoothed finite element method (NS-FEM) for upper bound solutions to solid mechanics problems. Computers and Structures; 87: 14-26.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H (2009) Additional properties of the node-based smoothed finite element method (NS-FEM) for solid mechanics problems. International Journal of Computational Methods, 6(4): 633-666.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H, Nguyen-Tran C (2010) Adaptive analysis using the node-based smoothed finite element method (NS-FEM). International Journal for Numerical Methods in Biomedical Engineering; in press, doi: 10.1002/cnm.1291.
% Liu GR, Nguyen-Thoi T, Lam KY (2009) An edge-based smoothed finite element method (ES-FEM) for static, free and forced vibration analyses in solids. Journal of Sound and Vibration; 320: 1100-1130.
% Nguyen-Xuan H, Liu GR, Thai-Hoang C, Nguyen-Thoi T (2009) An edge-based smoothed finite element method with stabilized discrete shear gap technique for analysis of Reissner-Mindlin plates. Computer Methods in Applied Mechanics and Engineering;  199: 471-489.
% Nguyen-Xuan H, Liu GR, Nguyen-Thoi T, Nguyen-Tran C (2009) An edge-based smoothed finite element method for analysis of two-dimensional piezoelectric structures, Smart Materials and Structures 18 (6). no. 065015.
% Nguyen-Thoi T, Liu GR, Vu-Do HC, Nguyen-Xuan H (2009) An edge-based smoothed finite element method (ES-FEM) for visco-elastoplastic analyses of 2D solids using triangular mesh. Computational Mechanics; 45: 23-44.
% Nguyen-Thoi T, Liu GR, Nguyen-Xuan H (2010) An n-sided polygonal edge-based smoothed finite element method (nES-FEM) for solid mechanics. International Journal for Numerical Methods in Biomedical Engineering; doi:10.1002/cnm.1375.
% Liu GR, Nguyen-Thoi T, Lam KY (2008) A novel Alpha Finite Element Method (?FEM) for exact solution to mechanics problems using triangular and tetrahedral elements. Computer Methods in Applied Mechanics and Engineering; 197: 3883-3897.
%--------------------------------------------------------------------------

clear
format long
global ndof sdof edof nel nnel nnode emodule poisson fload
global gcoord ele_nods matmtx example method

%----------------------------------------------%
%             1. PREPROCESSOR PHASE            %
%----------------------------------------------%
%----------------------------------------------%
%            1.1 Choose example                % 
%----------------------------------------------%
clc
fprintf('Choose example \n')
fprintf('1. cantilever \n')
fprintf('2. platehole \n')
option1=input('Option (1 or 2) = ');

if option1 == 1
    example = 'cantilever';
elseif option1 == 2
    example = 'platehole';
end

%------------------------------------------%
%      1.2 Choose numerical method         % 
%------------------------------------------%
clc
fprintf('Choose numerical method \n')
fprintf('1.FEM_T3 (FEM using triangular elements) \n')
fprintf('2.FEM_Q4 (FEM using quadrilateral elements) \n')
fprintf('3.CSFEM_Q4 (Cell-based smoothed FEM using quadrilateral elements) \n')
fprintf('4.NSFEM_T3 (Node-based smoothed FEM using triangular elements) \n')
fprintf('5.NSFEM_Q4 (Node-based smoothed FEM using quadrilateral elements) \n')
fprintf('6.ESFEM_T3 (Edge-based smoothed FEM using triangular elements) \n')
fprintf('7.alpha_FEM_T3 (Alpha-FEM using triangular elements) \n')

option2=input('Option (1,2,... or 7) = ');

if option2==1
    method = 'FEM_T3';          % FEM using triangular (T3) elements
      nnel = 3;                 % Number of nodes per element 
elseif option2==2
    method = 'FEM_Q4';          % FEM using quadrilateral (Q4) elements
      nnel = 4;                 % Number of nodes per element 
    nglx=2; ngly=2;             % 2x2 Gauss-Legendre quadrature
    nglxy=nglx*ngly;            % Number of sampling points per element
elseif option2==3
    method = 'CSFEM_Q4';        % Cell-based smoothed FEM using Q4 elements
      nnel = 4;                 % Number of nodes per element 
       nSD = input('Number of smoothing cells per element (1,2,3,4,8,16) = ');                 
elseif option2==4
    method = 'NSFEM_T3';        % Node-based smoothed FEM using T3 elements
      nnel = 3;                 % Number of nodes per element 
elseif option2==5
    method = 'NSFEM_Q4';        % Node-based smoothed FEM using Q4 elements
      nnel = 4;                 % Number of nodes per element 
elseif option2==6
    method = 'ESFEM_T3';        % Edge-based smoothed FEM using T3 elements
      nnel = 3;                 % Number of nodes per element 
elseif option2==7
    method = 'alpha_FEM_T3';    % alpha-FEM using triangular elements
      nnel = 3;                 % Number of nodes per element 
     alpha = input('Value of parameter alpha = (0<= alpha <=1) ');  % Parameter alpha   
end

%------------------------------------------%
%  1.3 Input the initial data of example   % 
%------------------------------------------%
switch example 
    case 'cantilever'    
        global lengthx lengthy nx ny air
        nx=input('Number of element in x-axis (16,24,32,40,48,...) = ');    
        ny=input('Number of element in y-axis (4,6,8,10,12,...) = ');        
        %air=input('Irregular factor of mesh (0.0,...,0.5) = ' );   
        air=0.0;             % irregular factor of mesh
        type=1;              % type=1 - plane stress analysis
                             % type=2 - plane strain analysis
                             % type=3 - axisymmetric analysis
                             % type=4 - three dimensional analysis
        lengthx=48;          % length of x-axis side of problem
        lengthy=12;          % length of y-axis side of problem   
        fload=1000;          % the total load 
        emodule=3e7;         % Elastic modulus
        poisson=0.3;         % Poisson's ratio 
        if nnel == 3
            nel = 2*nx*ny;   % Total number of element in system
        elseif nnel == 4
            nel = nx*ny;     % Total number of element in system
        end
        nnode=(nx+1)*(ny+1);       % total number of nodes in system    
    case 'platehole'
        global nx ny Ra A
        nx=input('Number of element in x-axis (8,12,16,20,24,...) = ');    
        ny=input('Number of element in y-axis (8,12,16,20,24,...) = ');    
        Ra=1; A=5;          % geometric dimensions of plate hole problem
        type=2;             % type=1 - plane stress analysis
                            % type=2 - plane strain analysis
                            % type=3 - axisymmetric analysis
                            % type=4 - three dimensional analysis
        emodule=1e3;        % elastic modulus
        poisson=0.3;        % Poisson's ratio
        if nnel == 3
            nel = 2*nx*ny;  % Total number of element in system
        elseif nnel == 4
            nel = nx*ny;    % Total number of element in system
        end
        nnode=(nx+1)*(ny+1);       % total number of nodes in system   
end

%----------------------------------------------%
%  1.4 Compute necessary data from input data  % 
%----------------------------------------------%
ndof=2;                    % number of degrees of freedom (dofs) per node 
sdof=nnode*ndof;           % total dofs in system
edof=nnel*ndof;            % Dofs per element        
matmtx=get_matmtx(type);   % matrix of material constants
disp=sparse(sdof,1);	   % system displacement vector

% initial data of geometry, boundary condition and force vector
[gcoord,ele_nods,bcdof,bcval,ff]=get_initdata;   

%-----------------------------------------------%
%             2. SOLUTION PHASE                 %
%-----------------------------------------------%
%-----------------------------------------------%
%      2.1 Compute the stiffness matrix         % 
%-----------------------------------------------%
switch method 
    case 'FEM_T3' 
        [K]=cal_K_FEM_T3;          
    case 'FEM_Q4' 
        [K]=cal_K_FEM_Q4(nglx,ngly);               
    case 'CSFEM_Q4'               
        [K]=cal_K_CSFEM_Q4(nSD);
        %[K]=cal_K_CSFEM_Q4_selective(nSD);     % for selective scheme
    case 'NSFEM_T3' 
        % find adjacent elements of each node
        [nod_adjele]=get_nod_adjele;     
        % Compute the area of SD associated with node and element areas
        [area_nod,area_T3] = cal_area_nod_T3(nod_adjele); 
        % Scheme1: Computed along the boundary of SD associated with node
        %[K]=cal_K_NSFEM_T3_boun(nod_adjele,area_nod);   
        % Scheme2: using average of Be matrices of adjacent elements of each node
        [K]=cal_K_NSFEM_T3_aver(nod_adjele,area_nod,area_T3);  
    case 'NSFEM_Q4' 
        % find adjacent elements of each node
        [nod_adjele]=get_nod_adjele;
        [area_nod] = cal_area_nod_Q4; % Compute integration domain for each node
        [K]=cal_K_NSFEM_Q4(nod_adjele,area_nod);
    case 'ESFEM_T3'          
        [edge_data,ele_edgs]=get_edge_data;
        [area_edg,area_T3] = cal_area_edge_T3(edge_data);
        % Scheme1: Computed along the boundary of SD associated with edge
        %[K]=cal_K_ESFEM_T3_boun(edge_data,area_edg);
        % Scheme2: using average of Be matrices of adjacent elements of each edge
        [K]=cal_K_ESFEM_T3_aver(edge_data,area_edg,area_T3); 
        %[K]=cal_K_ESFEM_T3_selective(edge_data,area_T3,area_edg); % for selective scheme
    case 'alpha_FEM_T3' 
        [K1]=cal_K_FEM_T3; 
        % find adjacent elements of each node
        [nod_adjele]=get_nod_adjele;    
        % Compute the area of SD associated with node and element areas
        [area_nod,area_T3] = cal_area_nod_T3(nod_adjele); 
        % Scheme1: Computed along the boundary of SD associated with node
        %[K2]=cal_K_NSFEM_T3_boun(nod_adjele,area_nod);   
        % Scheme2: using average of Be matrices of adjacent elements of each node
        [K2]=cal_K_NSFEM_T3_aver(nod_adjele,area_nod,area_T3);  
        K = alpha^2*K1 + (1-alpha^2)*K2;       
end    

%-----------------------------------------------%
%       2.2 Apply the boundary condition        %
%-----------------------------------------------%
K1=K;   % Save K1=K (before applying bcdof) to compute strain energy E 
[K,ff]=apply_bcdof(K,ff,bcdof,bcval);    

%-----------------------------------------------------------------------%
% 2.3 Solve the system of equations to obtain nodal displacement vector %
%-----------------------------------------------------------------------%
disp=K\ff;
clear K ff 

%-----------------------------------------------%
%              3. POSTPROCESSOR PHASE           %
%-----------------------------------------------%
%-------------------------------------%
% 3.1 compute strain energy of system %
%-------------------------------------%
energy = 0.5*disp'*K1*disp;
clear K1

%--------------------------------------------------------------------%
% 3.2 compute error norms: displacement, energy, and recovery energy %
%--------------------------------------------------------------------%
norm_E=0;
switch method 
    case 'FEM_T3' 
         [stress_nod,stress_ele]=cal_stress_FEM_T3(disp);         
         nGauss=7;     % Gauss-Legendre quadrature                      
         [norm_disp,norm_E,norm_Ereco]=cal_norms_FEM_T3(nGauss,disp,stress_nod,stress_ele);             
    case 'FEM_Q4' 
         [stress_nod]=cal_stress_nod_FEM_Q4(nglx,ngly,disp);
         nglx=3; ngly=3;     %3x3 Gauss-Legendre quadrature               
         [norm_disp,norm_E,norm_Ereco]=cal_norms_FEM_Q4(nglx,ngly,disp,stress_nod);
    case 'CSFEM_Q4' 
         [stress_nod]=cal_stress_nod_CSFEM_Q4(nSD,disp);
         nglx=3; ngly=3;     %3x3 Gauss-Legendre quadrature        
         [norm_disp,norm_Ereco]=cal_norms_CSFEM_Q4(nglx,ngly,disp,stress_nod);                
    case 'NSFEM_T3'
         %[stress_nod]=cal_stress_nod_NSFEM_T3_boun(nod_adjele,disp,area_nod);                               
         [stress_nod]=cal_stress_nod_NSFEM_T3_aver(nod_adjele,disp,area_nod,area_T3);
         nGauss=7;     % Gauss-Legendre quadrature               
         [norm_disp,norm_Ereco]=cal_norms_NSFEM_T3(nGauss,disp,stress_nod);
    case 'NSFEM_Q4'
         [stress_nod]=cal_stress_nod_NSFEM_Q4(nod_adjele,disp,area_nod);                               
         nglx=3; ngly=3;     %3x3 Gauss-Legendre quadrature               
         [norm_disp,norm_Ereco]=cal_norms_CSFEM_Q4(nglx,ngly,disp,stress_nod);               
    case 'ESFEM_T3'                    
         [stress_edg] = cal_stress_edg_ESFEM_T3_aver(edge_data,disp,area_edg,area_T3); 
         %[stress_edg] = cal_stress_edg_ESFEM_T3_boun(edge_data,area_edg,disp);    
         [nod_adjedg]=get_nod_adjedg(edge_data);                              
         [stress_nod]=cal_stress_nod_ESFEM_T3(nod_adjedg,stress_edg,area_edg);                                                
         nGauss=7;     % Gauss-Legendre quadrature               
         % recovery the first order stress
         [norm_disp,norm_Ereco]=cal_norms_ESFEM_T3(nGauss,disp,stress_nod,stress_edg,ele_edgs,3);
    case 'alpha_FEM_T3' 
         % [stress_nod]=cal_stress_nod_NSFEM_T3_boun(nod_adjele,disp,area_nod);                               
         [stress_nod]=cal_stress_nod_NSFEM_T3_aver(nod_adjele,disp,area_nod,area_T3);
         nGauss=7;     % Gauss-Legendre quadrature               
         [norm_disp,norm_Ereco]=cal_norms_NSFEM_T3(nGauss,disp,stress_nod);    
end

%------------------------------------------------------------------------%
% 3.3 Output error norms and plot the results of displacement and stress %
%------------------------------------------------------------------------%
output_results_2D(energy,norm_disp,norm_E,norm_Ereco,disp,stress_nod);

